# This file is part of the submission of the Chair for Computer Aided
# Medical Procedures, Technische Universitaet Muenchen, Germany to the
# Prostate Cancer DREAM Challenge 2015.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
cimport cython

import numpy as np
cimport numpy as cnp
from scipy.sparse import csr_matrix

cnp.import_array()


@cython.wraparound(False)
@cython.boundscheck(False)
def create_difference_matrix(cnp.npy_uint8[:] event,
                             cnp.ndarray[cnp.npy_double, ndim=1] time,
                             object kind):
    cdef cnp.ndarray[cnp.npy_intp, ndim=1] order
    cdef cnp.ndarray[cnp.npy_int8, ndim=1] values
    cdef cnp.ndarray[cnp.npy_intp, ndim=1] columns
    cdef cnp.npy_intp max_size
    cdef cnp.npy_intp k
    cdef cnp.npy_intp n_samples = event.shape[0]
    cdef cnp.ndarray[cnp.npy_intp, ndim=1] indptr
    cdef object ret

    order = cnp.PyArray_ArgSort(time, 0, cnp.NPY_MERGESORT)

    if kind == "all":
        max_size = n_samples * n_samples * 2
        values = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INT8, 0)
        columns = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INTP, 0)

        k = create_difference_matrix_full(event, order, values, columns)
    elif kind == "nearest":
        max_size = n_samples * 2
        values = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INT8, 0)
        columns = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INTP, 0)

        k = create_difference_matrix_nearest_neighbor(event, order, values, columns)
    elif kind == "next":
        max_size = n_samples * 2
        values = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INT8, 0)
        columns = cnp.PyArray_EMPTY(1, &max_size, cnp.NPY_INTP, 0)

        k = create_difference_matrix_direct_neighbor(event, order, values, columns)
    else:
        raise ValueError("pairs must be one of (all|nearest|next)")

    cdef cnp.PyArray_Dims new_dim
    new_dim.ptr = &k
    new_dim.len = 1

    ret = cnp.PyArray_Resize(values, &new_dim, 1, cnp.NPY_CORDER)
    if ret is not None:  # returns NULL on error
        return

    ret = cnp.PyArray_Resize(columns, &new_dim, 1, cnp.NPY_CORDER)
    if ret is not None:  # returns NULL on error
        return

    indptr = cnp.PyArray_Arange(0, k + 1, 2, cnp.NPY_INTP)
    D = csr_matrix((values, columns, indptr), shape=(k // 2, n_samples), copy=False, dtype=np.int8)

    return D


@cython.wraparound(False)
@cython.cdivision(True)
@cython.boundscheck(False)
cdef inline void set_entries(cnp.npy_intp[:] columns,
                             cnp.npy_int8[:] values,
                             cnp.npy_intp k,
                             cnp.npy_intp i, cnp.npy_intp j) nogil:
    """Create sparse matrix with sorted indices"""
    if i < j:
        columns[k] = i
        values[k] = 1
        columns[k + 1] = j
        values[k + 1] = -1
    else:
        columns[k] = j
        values[k] = -1
        columns[k + 1] = i
        values[k + 1] = 1


@cython.wraparound(False)
@cython.cdivision(True)
@cython.boundscheck(False)
cdef cnp.npy_intp create_difference_matrix_direct_neighbor(cnp.npy_uint8[:] event,
                                              cnp.npy_intp[:] o,
                                              cnp.npy_int8[:] values,
                                              cnp.npy_intp[:] columns) nogil:
    """Only compare against direct nearest neighbor according to time"""
    cdef cnp.npy_intp n_samples = event.shape[0]
    cdef cnp.npy_intp i
    cdef cnp.npy_intp j = 0
    cdef cnp.npy_intp k = 0
    cdef cnp.npy_intp k1
    cdef cnp.npy_intp k2

    for i in range(1, n_samples):
        while j < i:
            if event[o[j]]:
                set_entries(columns, values, k, o[i], o[j])
                k += 2
            j += 1

    return k


@cython.wraparound(False)
@cython.cdivision(True)
@cython.boundscheck(False)
cdef cnp.npy_intp create_difference_matrix_nearest_neighbor(cnp.npy_uint8[:] event,
                                               cnp.npy_intp[:] o,
                                               cnp.npy_int8[:] values,
                                               cnp.npy_intp[:] columns) nogil:
    """Only considers comparable pairs (i, j) where j is uncensored sample
    with highest survival time smaller than y_i"""
    cdef cnp.npy_intp n_samples = event.shape[0]
    cdef cnp.npy_intp k = 0
    cdef cnp.npy_intp i, j

    for i in range(1, n_samples):
        j = i - 1
        while j >= 0:
            if event[o[j]]:
                set_entries(columns, values, k, o[i], o[j])
                k += 2
                break
            j -= 1

    return k


@cython.wraparound(False)
@cython.cdivision(True)
@cython.boundscheck(False)
cdef cnp.npy_intp create_difference_matrix_full(cnp.npy_uint8[:] event,
                                   cnp.npy_intp[:] o,
                                   cnp.npy_int8[:] values,
                                   cnp.npy_intp[:] columns) nogil:
    """Considers all possible comparable pairs"""
    cdef cnp.npy_intp n_samples = event.shape[0]
    cdef cnp.npy_intp i, j
    cdef cnp.npy_intp k = 0

    for i in range(1, n_samples):
        for j in range(i):
            if event[o[j]]:
                set_entries(columns, values, k, o[i], o[j])
                k += 2

    return k



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numpy
from sklearn.linear_model import Ridge

from ..base import SurvivalAnalysisMixin
from ..nonparametric import ipc_weights
from ..util import check_arrays_survival


class IPCRidge(Ridge, SurvivalAnalysisMixin):
    """Accelerated failure time model with inverse probability of censoring weights.

    This model assumes a regression model of the form

    .. math::

        \\log y = \\beta_0 + \\mathbf{X} \\beta + \\epsilon

    L2-shrinkage is applied to the coefficients :math:`\\beta` and
    each sample is weighted by the inverse probability of censoring
    to account for right censoring (under the assumption that
    censoring is independent of the features, i.e., random censoring).

    Parameters
    ----------
    alpha : {float, array-like}
        shape = [n_targets]
        Small positive values of alpha improve the conditioning of the problem
        and reduce the variance of the estimates.
    """
    def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
                 copy_X=True, max_iter=None, tol=1e-3, solver="auto"):
        super(IPCRidge, self).__init__(alpha=alpha, fit_intercept=fit_intercept,
                                       normalize=normalize, copy_X=copy_X,
                                       max_iter=max_iter, tol=tol, solver=solver)

    def fit(self, X, y):
        """Build an accelerated failure time model.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix.

        y : structured array, shape = [n_samples]
            A structured array containing the binary event indicator
            as first field, and time of event or time of censoring as
            second field.

        Returns
        -------
        self
        """
        X, event, time = check_arrays_survival(X, y)

        weights = ipc_weights(event, time)
        super().fit(X, numpy.log(time), sample_weight=weights)

        return self

    def predict(self, X):
        """Predict using the linear accelerated failure time model.

        Parameters
        ----------
        X : {array-like, sparse matrix}, shape = [n_samples, n_features]
            Samples.

        Returns
        -------
        C : array, shape = (n_samples,)
            Returns predicted values on original scale (NOT log scale).
        """
        return numpy.exp(super().predict(X))

    def score(self, X, y, sample_weight=None):
        return SurvivalAnalysisMixin.score(self, X, y)



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numbers
from sklearn.utils.metaestimators import _safe_split


def _fit_and_score(est, x, y, scorer, train_index, test_index, parameters, fit_params, predict_params):
    """Train survival model on given data and return its score on test data"""
    X_train, y_train = _safe_split(est, x, y, train_index)
    train_params = fit_params.copy()

    # Training
    est.set_params(**parameters)
    est.fit(X_train, y_train, **train_params)

    # Testing
    test_predict_params = predict_params.copy()
    X_test, y_test = _safe_split(est, x, y, test_index, train_index)

    score = scorer(est, X_test, y_test, **test_predict_params)
    if not isinstance(score, numbers.Number):
        raise ValueError("scoring must return a number, got %s (%s) instead."
                         % (str(score), type(score)))

    return score



import os
import tempfile
import warnings
from io import StringIO

import numpy
import pandas
import pandas.util.testing as tm
from numpy.testing import TestCase, assert_array_equal, assert_array_almost_equal, run_module_suite

from sksurv.datasets import *
from sksurv.io import writearff

ARFF_CATEGORICAL_INDEX_1 = """@relation arff_categorical_index
@attribute index {SampleOne,SampleTwo,SampleThree,SampleFour}
@attribute value real
@attribute label {no,yes}
@attribute size {small,medium,large}
@data
SampleOne,15.1,yes,medium
SampleTwo,13.8,no,large
SampleThree,-0.2,yes,small
SampleFour,2.453,yes,large
"""

ARFF_CATEGORICAL_INDEX_2 = """@relation arff_categorical_index
@attribute index {ASampleOne,ASampleTwo,ASampleThree,ASampleFour,ASampleFive}
@attribute value real
@attribute label {yes,no}
@attribute size {small,medium,large}
@data
ASampleOne,1.51,no,small
ASampleTwo,1.38,no,small
ASampleThree,-20,yes,large
ASampleFour,245.3,yes,small
ASampleFive,3.14,no,large
"""


def _make_features(n_samples, n_features, seed):
    rnd = numpy.random.RandomState(seed)
    return rnd.randn(n_samples, n_features)


def _make_survival_data(n_samples, n_features, seed):
    rnd = numpy.random.RandomState(seed)

    x = _make_features(n_samples, n_features, seed)
    event = rnd.binomial(1, 0.2, n_samples)
    time = rnd.exponential(25, size=n_samples)
    return x, event, time


def _make_classification_data(n_samples, n_features, n_classes, seed):
    rnd = numpy.random.RandomState(seed)

    x = _make_features(n_samples, n_features, seed)
    y = rnd.binomial(n_classes - 1, 0.2, 100)
    return x, y


class TestGetXy(TestCase):
    def test_get_x_y_survival(self):
        x, event, time = _make_survival_data(100, 10, 0)
        columns = ["V{}".format(i) for i in range(10)] + ["event", "time"]
        dataset = pandas.DataFrame(numpy.column_stack((x, event, time)), columns=columns)

        attr_labels = ["event", "time"]

        x_test, y_test = get_x_y(dataset, attr_labels, pos_label=1, survival=True)

        self.assertTupleEqual(y_test.dtype.names, ("event", "time"))

        assert_array_equal(y_test["event"].astype(numpy.uint32),
                           event.astype(numpy.uint32))
        assert_array_almost_equal(y_test["time"], time)

        assert_array_equal(x, x_test)

    def test_get_x_y_survival_no_label(self):
        x = _make_features(100, 10, 0)

        columns = ["V{}".format(i) for i in range(10)]
        dataset = pandas.DataFrame(x, columns=columns)

        attr_labels = [None, None]

        x_test, y_test = get_x_y(dataset, attr_labels, pos_label=1, survival=True)

        self.assertEqual(y_test, None)
        assert_array_equal(x, x_test)

    def test_get_x_y_survival_too_many_labels(self):
        x, event, time = _make_survival_data(100, 10, 0)

        columns = ["V{}".format(i) for i in range(10)] + ["event", "time"]
        dataset = pandas.DataFrame(numpy.column_stack((x, event, time)), columns=columns)

        attr_labels = ["event", "time", "random"]
        self.assertRaisesRegex(ValueError, "expected sequence of length two for attr_labels, but got 3",
                               get_x_y, dataset, attr_labels, pos_label=1, survival=True)

    def test_get_x_y_survival_too_little_labels(self):
        x, event, time = _make_survival_data(100, 10, 0)

        columns = ["V{}".format(i) for i in range(10)] + ["event", "time"]
        dataset = pandas.DataFrame(numpy.column_stack((x, event, time)), columns=columns)

        self.assertRaisesRegex(ValueError, "expected sequence of length two for attr_labels, but got 1",
                               get_x_y, dataset, ["event"], pos_label=1, survival=True)

        self.assertRaisesRegex(ValueError, "expected sequence of length two for attr_labels, but got 0",
                               get_x_y, dataset, [], pos_label=1, survival=True)

    def test_get_x_y_survival_no_pos_label(self):
        x, event, time = _make_survival_data(100, 10, 0)

        columns = ["V{}".format(i) for i in range(10)] + ["event", "time"]
        dataset = pandas.DataFrame(numpy.column_stack((x, event, time)), columns=columns)

        self.assertRaisesRegex(ValueError, "pos_label needs to be specified if survival=True",
                               get_x_y, dataset, ["event", "time"], survival=True)

    def test_get_x_y_classification(self):
        x, label = _make_classification_data(100, 10, 6, 0)

        columns = ["V{}".format(i) for i in range(10)] + ["class_label"]
        dataset = pandas.DataFrame(numpy.column_stack((x, label)), columns=columns)

        attr_labels = ["class_label"]

        x_test, y_test = get_x_y(dataset, attr_labels, survival=False)

        self.assertEqual(y_test.ndim, 2)
        assert_array_equal(y_test.values.ravel(), label)
        assert_array_equal(x_test, x)

    def test_get_x_y_classification_no_label(self):
        x = _make_features(100, 10, 0)

        columns = ["V{}".format(i) for i in range(10)]
        dataset = pandas.DataFrame(x, columns=columns)

        x_test, y_test = get_x_y(dataset, None, survival=False)

        self.assertEqual(y_test, None)
        assert_array_equal(x_test, x)


class TestLoadDatasets(TestCase):

    def assert_structured_array_dtype(self, arr, event, time, num_events):
        self.assertTupleEqual(arr.dtype.names, (event, time))
        self.assertTrue(numpy.issubdtype(arr.dtype.fields[event][0], numpy.bool_))
        self.assertTrue(numpy.issubdtype(arr.dtype.fields[time][0], numpy.float_))
        self.assertEqual(arr[event].sum(), num_events)

    def test_load_whas500(self):
        x, y = load_whas500()
        self.assertTupleEqual(x.shape, (500, 14))
        self.assertTupleEqual(y.shape, (500,))
        self.assert_structured_array_dtype(y, 'fstat', 'lenfol', 215)

    def test_load_gbsg2(self):
        x, y = load_gbsg2()
        self.assertTupleEqual(x.shape, (686, 8))
        self.assertTupleEqual(y.shape, (686,))
        self.assert_structured_array_dtype(y, 'cens', 'time', 299)

    def test_load_veterans_lung_cancer(self):
        x, y = load_veterans_lung_cancer()
        self.assertTupleEqual(x.shape, (137, 6))
        self.assertTupleEqual(y.shape, (137,))
        self.assert_structured_array_dtype(y, 'Status', 'Survival_in_days', 128)

    def test_load_aids(self):
        x, y = load_aids(endpoint="aids")
        self.assertTupleEqual(x.shape, (1151, 11))
        self.assertTupleEqual(y.shape, (1151,))
        self.assert_structured_array_dtype(y, 'censor', 'time', 96)
        self.assertFalse("censor_d" in x.columns)
        self.assertFalse("time_d" in x.columns)

        x, y = load_aids(endpoint="death")
        self.assertTupleEqual(x.shape, (1151, 11))
        self.assertTupleEqual(y.shape, (1151,))
        self.assert_structured_array_dtype(y, 'censor_d', 'time_d', 26)
        self.assertFalse("censor" in x.columns)
        self.assertFalse("time" in x.columns)

        self.assertRaisesRegex(ValueError, "endpoint must be 'aids' or 'death'",
                               load_aids, endpoint="foobar")

    def test_load_breast_cancer(self):
        x, y = load_breast_cancer()
        self.assertTupleEqual(x.shape, (198, 80))
        self.assertTupleEqual(y.shape, (198,))
        self.assert_structured_array_dtype(y, 'e.tdm', 't.tdm', 51)


def _make_and_write_data(fp, n_samples, n_features, with_index, with_labels, seed, column_prefix="V"):
    x, event, time = _make_survival_data(n_samples, n_features, seed)

    columns = ["{}{}".format(column_prefix, i) for i in range(n_features)]
    if with_labels:
        columns += ["event", "time"]
        arr = numpy.column_stack((x, event, time))
    else:
        arr = x

    if with_index:
        index = numpy.arange(n_samples, dtype=numpy.float_)
        numpy.random.RandomState(0).shuffle(index)
    else:
        index = None

    dataset = pandas.DataFrame(arr, index=index, columns=columns)
    dataset.index.name = "index"

    writearff(dataset, fp, index=with_index)
    return dataset


class TestLoadArffFile(TestCase):

    def assert_x_equal(self, x_true, x_train):
        tm.assert_index_equal(x_true.columns, x_train.columns, exact=True)
        tm.assert_index_equal(x_true.index, x_train.index, exact=True)

        tm.assert_frame_equal(x_true, x_train,
                              check_index_type=False,
                              check_column_type=True,
                              check_names=False,
                              check_less_precise=True)

    def assert_y_equal(self, y_true, y_train):
        self.assertTupleEqual(y_train.dtype.names, ("event", "time"))

        assert_array_equal(y_train["event"].astype(numpy.uint32),
                           y_true["event"].values.astype(numpy.uint32))
        assert_array_almost_equal(y_train["time"], y_true["time"].values)

    def test_load_with_index(self):
        tmp = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        try:
            dataset = _make_and_write_data(tmp, 100, 10, True, True, 0)

            x_train, y_train, x_test, y_test = load_arff_files_standardized(
                tmp.name, ["event", "time"], 1, survival=True,
                standardize_numeric=False, to_numeric=False)

            self.assertEqual(x_test, None)
            self.assertEqual(y_test, None)

            cols = ["event", "time"]
            x_true = dataset.drop(cols, axis=1)

            self.assert_x_equal(x_true, x_train)
            self.assert_y_equal(dataset, y_train)
        finally:
            os.unlink(tmp.name)

    def test_load_with_categorical_index_1(self):
        fp = StringIO(ARFF_CATEGORICAL_INDEX_1)
        x_train, y_train, x_test, y_test = load_arff_files_standardized(
            fp, ["label"], pos_label="yes", survival=False,
            standardize_numeric=False, to_numeric=False)

        self.assertEqual(x_test, None)
        self.assertEqual(y_test, None)

        self.assertTupleEqual(x_train.shape, (4, 2))
        self.assertTupleEqual(y_train.shape, (4, 1))

        index = pandas.Index(['SampleOne', 'SampleTwo', 'SampleThree', 'SampleFour'],
                             name='index', dtype=object)
        tm.assert_index_equal(x_train.index, index, exact=True)

        label = pandas.Series(pandas.Categorical(["yes", "no", "yes", "yes"], categories=["no", "yes"], ordered=False),
                              name="label", index=index)
        tm.assert_series_equal(y_train["label"], label, check_exact=True)

        value = pandas.Series([15.1, 13.8, -0.2, 2.453], name="value", index=index)
        tm.assert_series_equal(x_train["value"], value, check_exact=True)

        size = pandas.Series(pandas.Categorical(["medium", "large", "small", "large"],
                                                categories=["small", "medium", "large"], ordered=False),
                             name="size", index=index)
        tm.assert_series_equal(x_train["size"], size, check_exact=True)

    def test_load_with_categorical_index_2(self):
        fp = StringIO(ARFF_CATEGORICAL_INDEX_2)
        x_train, y_train, x_test, y_test = load_arff_files_standardized(
            fp, ["label"], pos_label="yes", survival=False,
            standardize_numeric=False, to_numeric=False)

        self.assertEqual(x_test, None)
        self.assertEqual(y_test, None)

        self.assertTupleEqual(x_train.shape, (5, 2))
        self.assertTupleEqual(y_train.shape, (5, 1))

        index = pandas.Index(['ASampleOne', 'ASampleTwo', 'ASampleThree', 'ASampleFour', 'ASampleFive'],
                             name='index', dtype=object)
        tm.assert_index_equal(x_train.index, index, exact=True)

        label = pandas.Series(pandas.Categorical(["no", "no", "yes", "yes", "no"], categories=["yes", "no"], ordered=False),
                              name="label", index=index)
        tm.assert_series_equal(y_train["label"], label, check_exact=True)

        value = pandas.Series([1.51, 1.38, -20, 245.3, 3.14], name="value", index=index)
        tm.assert_series_equal(x_train["value"], value, check_exact=True)

        size = pandas.Series(pandas.Categorical(["small", "small", "large", "small", "large"],
                                                categories=["small", "medium", "large"], ordered=False),
                             name="size", index=index)
        tm.assert_series_equal(x_train["size"], size, check_exact=True)

    def test_load_train_and_test_with_labels(self):
        tmp_train = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        tmp_test = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        try:
            train_dataset = _make_and_write_data(tmp_train, 100, 10, True, True, 0)
            test_dataset = _make_and_write_data(tmp_test, 20, 10, True, True, 0)

            x_train, y_train, x_test, y_test = load_arff_files_standardized(
                tmp_train.name, ["event", "time"], 1, path_testing=tmp_test.name,
                survival=True, standardize_numeric=False, to_numeric=False)

            cols = ["event", "time"]

            x_true = train_dataset.drop(cols, axis=1)
            self.assert_x_equal(x_true, x_train)
            self.assert_y_equal(train_dataset, y_train)

            x_true = test_dataset.drop(cols, axis=1)
            self.assert_x_equal(x_true, x_test)
            self.assert_y_equal(test_dataset, y_test)
        finally:
            os.unlink(tmp_train.name)
            os.unlink(tmp_test.name)

    def test_load_train_and_test_with_categorical_index(self):
        fp_1 = StringIO(ARFF_CATEGORICAL_INDEX_1)
        fp_2 = StringIO(ARFF_CATEGORICAL_INDEX_2)
        x_train, y_train, x_test, y_test = load_arff_files_standardized(
            fp_1, ["label"], pos_label="yes", path_testing=fp_2, survival=False,
            standardize_numeric=False, to_numeric=False)

        self.assertTupleEqual(x_train.shape, (4, 2))
        self.assertTupleEqual(x_test.shape, (5, 2))
        self.assertTupleEqual(y_train.shape, (4, 1))
        self.assertTupleEqual(y_test.shape, (5, 1))

        # Check train data
        train_index = pandas.Index(['SampleOne', 'SampleTwo', 'SampleThree', 'SampleFour'],
                                   name='index', dtype=object)
        tm.assert_index_equal(x_train.index, train_index, exact=True)

        train_label = pandas.Series(
            pandas.Categorical(["yes", "no", "yes", "yes"], categories=["no", "yes"], ordered=False),
            name="label", index=train_index)
        tm.assert_series_equal(y_train["label"], train_label, check_exact=True)

        train_value = pandas.Series([15.1, 13.8, -0.2, 2.453], name="value", index=train_index)
        tm.assert_series_equal(x_train["value"], train_value, check_exact=True)

        train_size = pandas.Series(pandas.Categorical(["medium", "large", "small", "large"],
                                                      categories=["small", "medium", "large"], ordered=False),
                                   name="size", index=train_index)
        tm.assert_series_equal(x_train["size"], train_size, check_exact=True)

        # Check test data
        test_index = pandas.Index(['ASampleOne', 'ASampleTwo', 'ASampleThree', 'ASampleFour', 'ASampleFive'],
                                  name='index', dtype=object)
        tm.assert_index_equal(x_test.index, test_index, exact=True)

        test_label = pandas.Series(
            pandas.Categorical(["no", "no", "yes", "yes", "no"], categories=["yes", "no"], ordered=False),
            name="label", index=test_index)
        tm.assert_series_equal(y_test["label"], test_label, check_exact=True)

        test_value = pandas.Series([1.51, 1.38, -20, 245.3, 3.14], name="value", index=test_index)
        tm.assert_series_equal(x_test["value"], test_value, check_exact=True)

        test_size = pandas.Series(pandas.Categorical(["small", "small", "large", "small", "large"],
                                                     categories=["small", "medium", "large"], ordered=False),
                                  name="size", index=test_index)
        tm.assert_series_equal(x_test["size"], test_size, check_exact=True)

    def test_load_train_and_test_no_labels(self):
        tmp_train = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        tmp_test = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        try:
            train_dataset = _make_and_write_data(tmp_train, 100, 10, True, True, 0)
            test_dataset = _make_and_write_data(tmp_test, 20, 10, True, False, 0)

            x_train, y_train, x_test, y_test = load_arff_files_standardized(
                tmp_train.name, ["event", "time"], 1, path_testing=tmp_test.name,
                survival=True, standardize_numeric=False, to_numeric=False)

            cols = ["event", "time"]

            x_true = train_dataset.drop(cols, axis=1)
            self.assert_x_equal(x_true, x_train)
            self.assert_y_equal(train_dataset, y_train)

            self.assert_x_equal(test_dataset, x_test)
            self.assertEqual(y_test, None)
        finally:
            os.unlink(tmp_train.name)
            os.unlink(tmp_test.name)

    def test_load_train_and_test_with_different_columns(self):
        tmp_train = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        tmp_test = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        try:
            _make_and_write_data(tmp_train, 100, 19, False, True, 0)
            _make_and_write_data(tmp_test, 20, 11, False, True, 0)

            with warnings.catch_warnings(record=True) as w:
                warnings.simplefilter("always")

                load_arff_files_standardized(tmp_train.name, ["event", "time"], 1,
                                             path_testing=tmp_test.name,
                                             survival=True,
                                             standardize_numeric=False, to_numeric=False)

                self.assertEqual(1, len(w))
                self.assertEqual("Restricting columns to intersection between training and testing data",
                                 str(w[0].message))

        finally:
            os.unlink(tmp_train.name)
            os.unlink(tmp_test.name)

    def test_load_train_and_test_columns_dont_intersect(self):
        tmp_train = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        tmp_test = tempfile.NamedTemporaryFile("w", suffix=".arff", delete=False)
        try:
            _make_and_write_data(tmp_train, 100, 19, True, True, 0, column_prefix="A")
            _make_and_write_data(tmp_test, 20, 11, True, True, 0, column_prefix="B")

            self.assertRaisesRegex(ValueError, "columns of training and test data do not intersect",
                                   load_arff_files_standardized, tmp_train.name, ["event", "time"], 1,
                                   path_testing=tmp_test.name,
                                   survival=True,
                                   standardize_numeric=False, to_numeric=False)

        finally:
            os.unlink(tmp_train.name)
            os.unlink(tmp_test.name)


if __name__ == '__main__':
    run_module_suite()



from os.path import join, dirname

import numpy
from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal

from sksurv.linear_model import IPCRidge
from sksurv.column import standardize
from sksurv.datasets import load_whas500
from sksurv.metrics import concordance_index_censored


class TestIPCRidge(TestCase):
    def setUp(self):
        x, self.y, =  load_whas500()
        self.x = standardize(x)

    def test_fit(self):
        model = IPCRidge()
        model.fit(self.x, self.y)

        self.assertAlmostEqual(model.intercept_, 5.8673567124629571)
        expected = numpy.array([0.168517, -0.249717, 2.18515, 0.536795, -0.514571, 0.091203,
                                0.613006, 0.480385, -0.055949, 0.238529, -0.127148, -0.144134,
                                -1.625041, -0.217469])
        assert_array_almost_equal(model.coef_, expected)

    def test_predict(self):
        model = IPCRidge()
        model.fit(self.x[:400], self.y[:400])

        x_test = self.x[400:]
        y_test = self.y[400:]
        p = model.predict(x_test)
        ci = concordance_index_censored(y_test['fstat'], y_test['lenfol'], -p)

        self.assertAlmostEqual(ci[0], 0.66925817946226107)
        self.assertEqual(ci[1], 2066)
        self.assertEqual(ci[2], 1021)
        self.assertEqual(ci[3], 0)
        self.assertEqual(ci[4], 6)

        self.assertEqual(model.score(x_test, y_test), 1.0 - ci[0])


if __name__ == '__main__':
    run_module_suite()



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numpy


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('bintrees', parent_package, top_path)

    config.add_extension('_binarytrees',
                         sources=['_binarytrees.cpp', 'binarytrees.cpp'],
                         include_dirs=[numpy.get_include()],
                         language='c++')

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from sklearn.utils import check_consistent_length, check_array

import numpy

__all__ = ['concordance_index_censored']


def concordance_index_censored(event_indicator, event_time, estimate):
    """Concordance index for right-censored data

    The concordance index is defined as the proportion of all comparable pairs
    in which the predictions and outcomes are concordant.

    Samples are comparable if for at least one of them an event occurred.
    If the estimated risk is larger for the sample with a higher time of
    event/censoring, the predictions of that pair are said to be concordant.
    If an event occurred for one sample and the other is known to be
    event-free at least until the time of event of the first, the second
    sample is assumed to *outlive* the first.
    When predicted risks are identical for a pair, 0.5 rather than 1 is added
    to the count of concordant pairs.
    A pair is not comparable if an event occurred for both of them at the same
    time or an event occurred for one of them but the time of censoring is
    smaller than the time of event of the first one.

    Parameters
    ----------
    event_indicator : array-like, shape = [n_samples,]
        Boolean array denotes whether an event occurred

    event_time : array-like, shape = [n_samples,]
        Array containing the time of an event or time of censoring

    estimate : array-like, shape = [n_samples,]
        Estimated risk of experiencing an event

    Returns
    -------
    cindex : float
        Concordance index

    concordant : int
        Number of concordant pairs

    discordant : int
        Number of discordant pairs

    tied_risk : int
        Number of pairs having tied estimated risks

    tied_time : int
        Number of pairs having an event at the same time

    References
    ----------
    .. [1] Harrell, F.E., Califf, R.M., Pryor, D.B., Lee, K.L., Rosati, R.A,
           "Multivariable prognostic models: issues in developing models,
           evaluating assumptions and adequacy, and measuring and reducing errors",
           Statistics in Medicine, 15(4), 361-87, 1996.
    """
    check_consistent_length(event_indicator, event_time, estimate)
    event_indicator = check_array(event_indicator, ensure_2d=False)
    event_time = check_array(event_time, ensure_2d=False)
    estimate = check_array(estimate, ensure_2d=False)

    if not numpy.issubdtype(event_indicator.dtype, numpy.bool_):
        raise ValueError(
            'only boolean arrays are supported as class labels for survival analysis, got {0}'.format(
                event_indicator.dtype))

    n_samples = len(event_time)
    if n_samples < 2:
        raise ValueError("Need a minimum of two samples")

    if not event_indicator.any():
        raise ValueError("All samples are censored")

    order = numpy.argsort(event_time)

    tied_time = 0
    comparable = {}
    for i in range(n_samples - 1):
        inext = i + 1
        j = inext
        time_i = event_time[order[i]]
        while j < n_samples and event_time[order[j]] == time_i:
            j += 1

        if event_indicator[order[i]]:
            mask = numpy.zeros(n_samples, dtype=bool)
            mask[inext:] = True
            if j - i > 1:
                # event times are tied, need to check for coinciding events
                event_at_same_time = event_indicator[order[inext:j]]
                mask[inext:j] = numpy.logical_not(event_at_same_time)
                tied_time += event_at_same_time.sum()
            comparable[i] = mask
        elif j - i > 1:
            # events at same time are comparable if at least one of them is positive
            mask = numpy.zeros(n_samples, dtype=bool)
            mask[inext:j] = event_indicator[order[inext:j]]
            comparable[i] = mask

    concordant = 0
    discordant = 0
    tied_risk = 0
    for ind, mask in comparable.items():
        est_i = estimate[order[ind]]
        event_i = event_indicator[order[ind]]

        est = estimate[order[mask]]

        if event_i:
            # an event should have a higher score
            con = (est < est_i).sum()
        else:
            # a non-event should have a lower score
            con = (est > est_i).sum()
        concordant += con

        tie = (est == est_i).sum()
        tied_risk += tie

        discordant += est.size - con - tie

    cindex = (concordant + 0.5 * tied_risk) / (concordant + discordant + tied_risk)
    return cindex, concordant, discordant, tied_risk, tied_time



from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal, assert_array_equal
import pandas.util.testing as tm
import pandas
import numpy

from sksurv.util import safe_concat


class TestUtil(TestCase):
    def test_concat_numeric(self):
        rnd = numpy.random.RandomState(14)
        a = pandas.Series(rnd.randn(100), name="col_A")
        b = pandas.Series(rnd.randn(100), name="col_B")

        expected_df = pandas.DataFrame.from_items(
            [(a.name, a), (b.name, b)]
        )

        actual_df = safe_concat((a, b), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    def test_concat_numeric_categorical(self):
        rnd = numpy.random.RandomState(14)
        a = pandas.Series(rnd.randn(100), name="col_A")
        b = pandas.Series(pandas.Categorical.from_codes(
            rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_B")

        expected_df = pandas.DataFrame.from_items(
            [(a.name, a), (b.name, b)]
        )

        actual_df = safe_concat((a, b), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    def test_concat_categorical(self):
        rnd = numpy.random.RandomState(14)
        a = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.6, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))])
        b = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.2, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))])

        expected_series = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                numpy.concatenate((a.col_A.cat.codes.values, b.col_A.cat.codes.values)),
                ["C1", "C2", "C3"]
            ))),
            ("col_B", numpy.concatenate((a.col_B.values, b.col_B.values)))
        ])
        expected_series.index = pandas.Index(a.index.tolist() + b.index.tolist())

        actual_series = safe_concat((a, b), axis=0)

        tm.assert_frame_equal(actual_series, expected_series)

    def test_concat_categorical_mismatch(self):
        rnd = numpy.random.RandomState(14)
        a = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.6, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))])
        b = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(3, 0.6, 100), ["C1", "C2", "C3", "C4"]), name="col_A")),
            ("col_B", rnd.randn(100))])

        self.assertRaisesRegex(ValueError, "categories for column col_A do not match",
                               safe_concat, (a, b), axis=0)

    def test_concat_dataframe_numeric_categorical(self):
        rnd = numpy.random.RandomState(14)
        numeric_df = pandas.DataFrame.from_items(
            [("col_A", rnd.randn(100)), ("col_B", rnd.randn(100))]
        )

        cat_series = pandas.Series(pandas.Categorical.from_codes(
            rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_C")

        expected_df = numeric_df.copy()
        expected_df["col_C"] = cat_series

        actual_df = safe_concat((numeric_df, cat_series), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    def test_concat_duplicate_columns(self):
        rnd = numpy.random.RandomState(14)
        numeric_df = pandas.DataFrame.from_items([
            ("col_N", rnd.randn(100)), ("col_B", rnd.randn(100)),
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(4, 0.2, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_A")),
        ])

        cat_df = pandas.DataFrame.from_items([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_A")),
            ("col_C", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(1, 0.6, 100), ["Yes", "No"]), name="col_C")),
        ])

        self.assertRaisesRegex(ValueError, "duplicate columns col_A",
                               safe_concat, (numeric_df, cat_df), axis=1)


if __name__ == '__main__':
    run_module_suite()



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numbers

import numpy
from scipy.stats import rankdata, kendalltau, spearmanr
from sklearn.base import clone, BaseEstimator
from sklearn.model_selection import check_cv
from sklearn.externals.joblib import Parallel, delayed

from .base import _fit_and_score
from .stacking import Stacking

__all__ = ["EnsembleSelection", "EnsembleSelectionRegressor", "MeanEstimator"]


def _corr_kendalltau(X):
    n_variables = X.shape[1]
    mat = numpy.empty((n_variables, n_variables), dtype=float)
    for i in range(n_variables):
        for j in range(i):
            v = kendalltau(X[:, i], X[:, j]).correlation
            mat[i, j] = v
            mat[j, i] = v
    return mat


class EnsembleAverage(BaseEstimator):
    def __init__(self, base_estimators, name=None):
        self.base_estimators = base_estimators
        self.name = name
        assert not hasattr(self.base_estimators[0], "classes_"),\
            "base estimator cannot be a classifier"

    def get_base_params(self):
        return self.base_estimators[0].get_params()

    def fit(self, X, y=None, **kwargs):
        return self

    def predict(self, X):
        prediction = numpy.zeros(X.shape[0])
        for est in self.base_estimators:
            prediction += est.predict(X)

        return prediction / len(self.base_estimators)


class MeanEstimator(BaseEstimator):
    def fit(self, X, y=None, **kwargs):
        return self

    def predict(self, X):
        return X.mean(axis=X.ndim - 1)


class MeanRankEstimator(BaseEstimator):
    def fit(self, X, y=None, **kwargs):
        return self

    def predict(self, X):
        # convert predictions of individual models into ranks
        ranks = numpy.apply_along_axis(rankdata, 0, X)
        # average predicted ranks
        return ranks.mean(axis=X.ndim - 1)


def _fit_and_score_fold(est, x, y, scorer, train_index, test_index, fit_params, idx, fold):
    score = _fit_and_score(est, x, y, scorer, train_index, test_index,
                           est.get_params(), fit_params, {})
    return idx, fold, score, est


def _predict(estimator, X, idx):
    return idx, estimator.predict(X)


def _score_regressor(estimator, X, y, idx):
    name_time = y.dtype.names[1]
    error = (estimator.predict(X).ravel() - y[name_time]) ** 2
    return idx, error


class BaseEnsembleSelection(Stacking):

    def __init__(self, meta_estimator, base_estimators, scorer=None, n_estimators=0.2,
                 min_score=0.66, correlation="pearson", min_correlation=0.6,
                 cv=None, n_jobs=1, verbose=0):
        super().__init__(meta_estimator=meta_estimator,
                         base_estimators=base_estimators)

        self.scorer = scorer
        self.n_estimators = n_estimators
        self.min_score = min_score
        self.correlation = correlation
        self.min_correlation = min_correlation
        self.cv = cv
        self.n_jobs = n_jobs
        self.verbose = verbose

        self._extra_params.extend(["scorer", "n_estimators", "min_score", "min_correlation",
                                   "cv", "n_jobs", "verbose"])

    def __len__(self):
        if hasattr(self, "fitted_models_"):
            return len(self.fitted_models_)
        else:
            return 0

    def _check_params(self):
        if self.n_estimators <= 0:
            raise ValueError("n_estimators must not be zero or negative")

        if self.n_estimators >= 1.0:
            self.n_estimators = int(self.n_estimators)
            if self.n_estimators > len(self.base_estimators):
                raise ValueError(
                    "n_estimators (%d) must not exceed number of base learners (%d)" % (self.n_estimators,
                                                                                        len(self.base_estimators)))

        if isinstance(self.n_estimators, numbers.Integral):
            self.n_estimators_ = self.n_estimators
        else:
            self.n_estimators_ = max(int(self.n_estimators * len(self.base_estimators)), 1)

        if not callable(self.scorer):
            raise TypeError("scorer is not callable")

        if not -1. <= self.min_correlation <= 1:
            raise ValueError("min_correlation must be in [-1; 1], but was %r" % self.min_correlation)

        if self.correlation == "pearson":
            self._corr_func = lambda x: numpy.corrcoef(x, rowvar=0)
        elif self.correlation == "kendall":
            self._corr_func = _corr_kendalltau
        elif self.correlation == "spearman":
            self._corr_func = lambda x: spearmanr(x, axis=0).correlation
        else:
            raise ValueError("correlation must be one of 'pearson', 'kendall', and 'spearman', "
                             "but got %r" % self.correlation)

    def _create_base_ensemble(self, out, n_estimators, n_folds):
        """For each base estimator collect models trained on each fold"""
        ensemble_scores = numpy.empty((n_estimators, n_folds))
        base_ensemble = numpy.empty_like(ensemble_scores, dtype=numpy.object)
        for model, fold, score, est in out:
            ensemble_scores[model, fold] = score
            base_ensemble[model, fold] = est

        return ensemble_scores, base_ensemble

    def _create_cv_ensemble(self, base_ensemble, idx_models_included, model_names=None):
        """For each selected base estimator, average models trained on each fold"""
        fitted_models = numpy.empty(len(idx_models_included), dtype=numpy.object)
        for i, idx in enumerate(idx_models_included):
            model_name = self.base_estimators[idx][0] if model_names is None else model_names[idx]
            avg_model = EnsembleAverage(base_ensemble[idx, :], name=model_name)
            fitted_models[i] = avg_model

        return fitted_models

    def _get_base_estimators(self, X):
        """Takes special care of estimators using custom kernel function

        Parameters
        ----------
        X : array, shape = [n_samples, n_features]
            Samples to pre-compute kernel matrix from.

        Returns
        -------
        base_estimators : list
            Same as `self.base_estimators`, expect that estimators with custom kernel function
            use ``kernel='precomputed'``.

        kernel_cache : dict
            Maps estimator name to kernel matrix. Use this for cross-validation instead of `X`.
        """
        base_estimators = []

        kernel_cache = {}
        kernel_fns = {}
        for i, (name, estimator) in enumerate(self.base_estimators):
            if hasattr(estimator, 'kernel') and callable(estimator.kernel):
                if not hasattr(estimator, '_get_kernel'):
                    raise ValueError(
                        'estimator %s uses a custom kernel function, but does not have a _get_kernel method' % name)

                kernel_mat = kernel_fns.get(estimator.kernel, None)
                if kernel_mat is None:
                    kernel_mat = estimator._get_kernel(X)
                    kernel_cache[i] = kernel_mat
                    kernel_fns[estimator.kernel] = kernel_mat

                kernel_cache[i] = kernel_mat

                # We precompute kernel, but only for training, for testing use original custom kernel function
                kernel_estimator = clone(estimator)
                kernel_estimator.set_params(kernel='precomputed')
                base_estimators.append((name, kernel_estimator))
            else:
                base_estimators.append((name, estimator))

        return base_estimators, kernel_cache

    def _restore_base_estimators(self, kernel_cache, out, X, cv):
        """Restore custom kernel functions of estimators for predictions"""
        train_folds = {fold: train_index for fold, (train_index, _) in enumerate(cv)}

        for idx, fold, _, est in out:
            if idx in kernel_cache:
                if not hasattr(est, 'fit_X_'):
                    raise ValueError(
                        'estimator %s uses a custom kernel function, '
                        'but does not have the attribute `fit_X_` after training' % self.base_estimators[idx][0])

                est.set_params(kernel=self.base_estimators[idx][1].kernel)
                est.fit_X_ = X[train_folds[fold]]

        return out

    def _fit_and_score_ensemble(self, X, y, cv, **fit_params):
        """Create a cross-validated model by training a model for each fold with the same model parameters"""
        fit_params_steps = self._split_fit_params(fit_params)

        folds = list(cv.split(X, y))

        # Take care of custom kernel functions
        base_estimators, kernel_cache = self._get_base_estimators(X)

        out = Parallel(
            n_jobs=self.n_jobs, verbose=self.verbose
        )(
            delayed(_fit_and_score_fold)(clone(estimator),
                                         X if i not in kernel_cache else kernel_cache[i],
                                         y,
                                         self.scorer,
                                         train_index, test_index,
                                         fit_params_steps[name],
                                         i, fold)
            for i, (name, estimator) in enumerate(base_estimators)
            for fold, (train_index, test_index) in enumerate(folds))

        if len(kernel_cache) > 0:
            out = self._restore_base_estimators(kernel_cache, out, X, folds)

        return self._create_base_ensemble(out, len(base_estimators), len(folds))

    def _add_diversity_score(self, scores, predictions):
        n_models = predictions.shape[1]

        cor = self._corr_func(predictions)
        assert cor.shape == (n_models, n_models)
        numpy.fill_diagonal(cor, 0)

        final_scores = scores.copy()
        diversity = numpy.apply_along_axis(
            lambda x: (n_models - numpy.sum(x >= self.min_correlation)) / n_models,
            0, cor)

        final_scores += diversity
        return final_scores

    def _fit(self, X, y, cv, **fit_params):
        raise NotImplementedError()

    def fit(self, X, y=None, **fit_params):
        """Fit ensemble of models

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Training data.

        y : array-like, optional
            Target data if base estimators are supervised.

        Returns
        -------
        self
        """
        self._check_params()

        cv = check_cv(self.cv, X)
        self._fit(X, y, cv, **fit_params)

        return self


class EnsembleSelection(BaseEnsembleSelection):
    """Ensemble selection for survival analysis that accounts for a score and correlations between predictions.

    The ensemble is pruned during training only according to the specified score (accuracy) and
    additionally for prediction according to the correlation between predictions (diversity).

    The hillclimbing is based on cross-validation to avoid having to create a separate validation set.

    Parameters
    ----------
    base_estimators : list
        List of (name, estimator) tuples (implementing fit/predict) that are
        part of the ensemble.

    scorer : callable
        Function with signature ``func(estimator, X_test, y_test, **test_predict_params)`` that evaluates the error
        of the prediction on the test data. The function should return a scalar value.
        *Larger* values of the score are assumed to be better.

    n_estimators : float or int
        If a float, the percentage of estimators in the ensemble to retain, if an int the
        absolute number of estimators to retain.

    min_score : float, optional, default = 0.66
        Threshold for pruning estimators based on scoring metric. After `fit`, only estimators
        with a score above `min_score` are retained.

    min_correlation : float, optional, default = 0.6
        Threshold for Pearson's correlation coefficient that determines when predictions of
        two estimators are significantly correlated.

    cv : int, a cv generator instance, or None
        The input specifying which cv generator to use. It can be an
        integer, in which case it is the number of folds in a KFold,
        None, in which case 3 fold is used, or another object, that
        will then be used as a cv generator. The generator has to ensure
        that each sample is only used once for testing.

    n_jobs : int, default 1
        Number of jobs to run in parallel.

    verbose : integer
        Controls the verbosity: the higher, the more messages.

    Attributes
    ----------
    `scores_` : ndarray, shape = [n_base_estimators,]
        Array of scores (relative to best performing estimator)

    `fitted_models_` : ndarray
        Selected models during training based on `scorer`.

    References
    ----------

    .. [1] Pölsterl, S., Gupta, P., Wang, L., Conjeti, S., Katouzian, A., and Navab, N.,
         "Heterogeneous ensembles for predicting survival of metastatic, castrate-resistant prostate cancer patients".
         F1000Research, vol. 5, no. 2676, 2016

    .. [2] Caruana, R., Munson, A., Niculescu-Mizil, A.
        "Getting the most out of ensemble selection". 6th IEEE International Conference on Data Mining, 828-833, 2006

    .. [3] Rooney, N., Patterson, D., Anand, S., Tsymbal, A.
        "Dynamic integration of regression models. International Workshop on Multiple Classifier Systems".
        Lecture Notes in Computer Science, vol. 3181, 164-173, 2004
    """

    def __init__(self, base_estimators, scorer=None, n_estimators=0.2,
                 min_score=0.2, correlation="pearson", min_correlation=0.6,
                 cv=None, n_jobs=1, verbose=0):
        super().__init__(meta_estimator=MeanRankEstimator(),
                         base_estimators=base_estimators,
                         scorer=scorer,
                         n_estimators=n_estimators,
                         min_score=min_score,
                         correlation=correlation,
                         min_correlation=min_correlation,
                         cv=cv,
                         n_jobs=n_jobs,
                         verbose=verbose)

    def _fit(self, X, y, cv, **fit_params):
        scores, base_ensemble = self._fit_and_score_ensemble(X, y, cv, **fit_params)
        self.fitted_models_, self.scores_ = self._prune_by_cv_score(scores, base_ensemble)

    def _prune_by_cv_score(self, scores, base_ensemble, model_names=None):
        mean_scores = scores.mean(axis=1)
        idx_good_models = numpy.flatnonzero(mean_scores >= self.min_score)
        if len(idx_good_models) == 0:
            raise ValueError("no base estimator exceeds min_score, try decreasing it")

        total_score = mean_scores[idx_good_models]
        max_score = total_score.max()
        total_score /= max_score

        fitted_models = self._create_cv_ensemble(base_ensemble, idx_good_models, model_names)

        return fitted_models, total_score

    def _prune_by_correlation(self, X):
        n_models = len(self.fitted_models_)

        out = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
            delayed(_predict)(est, X, i)
            for i, est in enumerate(self.fitted_models_))

        predictions = numpy.empty((X.shape[0], n_models), order="F")
        for i, p in out:
            predictions[:, i] = p

        if n_models > self.n_estimators_:
            final_scores = self._add_diversity_score(self.scores_, predictions)
            sorted_idx = numpy.argsort(-final_scores, kind="mergesort")

            selected_models = sorted_idx[:self.n_estimators_]
            return predictions[:, selected_models]

        return predictions

    def _predict_estimators(self, X):
        predictions = self._prune_by_correlation(X)
        return predictions


class EnsembleSelectionRegressor(BaseEnsembleSelection):
    """Ensemble selection for regression that accounts for the accuracy and correlation of errors.

    The ensemble is pruned during training according to estimators' accuracy and the correlation
    between prediction errors per sample. The accuracy of the *i*-th estimator defined as
    :math:`\\frac{ \\min_{i=1,\\ldots, n}(error_i) }{ error_i }`.
    In addition to the accuracy, models are selected based on the correlation between residuals
    of different models (diversity). The diversity of the *i*-th estimator is defined as
    :math:`\\frac{n-count}{n}`, where *count* is the number of estimators for whom the correlation
    of residuals exceeds `min_correlation`.

    The hillclimbing is based on cross-validation to avoid having to create a separate validation set.

    Parameters
    ----------
    base_estimators : list
        List of (name, estimator) tuples (implementing fit/predict) that are
        part of the ensemble.

    scorer : callable
        Function with signature ``func(estimator, X_test, y_test, **test_predict_params)`` that evaluates the error
        of the prediction on the test data. The function should return a scalar value.
        *Smaller* values of the score are assumed to be better.

    n_estimators : float or int
        If a float, the percentage of estimators in the ensemble to retain, if an int the
        absolute number of estimators to retain.

    min_score : float, optional, default = 0.66
        Threshold for pruning estimators based on scoring metric. After `fit`, only estimators
        with a accuracy above `min_score` are retained.

    min_correlation : float, optional, default = 0.6
        Threshold for Pearson's correlation coefficient that determines when residuals of
        two estimators are significantly correlated.

    cv : int, a cv generator instance, or None
        The input specifying which cv generator to use. It can be an
        integer, in which case it is the number of folds in a KFold,
        None, in which case 3 fold is used, or another object, that
        will then be used as a cv generator. The generator has to ensure
        that each sample is only used once for testing.

    n_jobs : int, default 1
        Number of jobs to run in parallel.

    verbose : integer
        Controls the verbosity: the higher, the more messages.

    Attributes
    ----------
    `scores_` : ndarray, shape = [n_base_estimators,]
        Array of scores (relative to best performing estimator)

    `fitted_models_` : ndarray
        Selected models during training based on `scorer`.

    References
    ----------

    .. [1] Pölsterl, S., Gupta, P., Wang, L., Conjeti, S., Katouzian, A., and Navab, N.,
         "Heterogeneous ensembles for predicting survival of metastatic, castrate-resistant prostate cancer patients".
         F1000Research, vol. 5, no. 2676, 2016

    .. [2] Caruana, R., Munson, A., Niculescu-Mizil, A.
        "Getting the most out of ensemble selection". 6th IEEE International Conference on Data Mining, 828-833, 2006

    .. [3] Rooney, N., Patterson, D., Anand, S., Tsymbal, A.
        "Dynamic integration of regression models. International Workshop on Multiple Classifier Systems".
        Lecture Notes in Computer Science, vol. 3181, 164-173, 2004
    """
    def __init__(self, base_estimators, scorer=None, n_estimators=0.2,
                 min_score=0.66, correlation="pearson", min_correlation=0.6,
                 cv=None, n_jobs=1, verbose=0):
        super().__init__(meta_estimator=MeanEstimator(),
                         base_estimators=base_estimators,
                         scorer=scorer,
                         n_estimators=n_estimators,
                         min_score=min_score,
                         correlation=correlation,
                         min_correlation=min_correlation,
                         cv=cv,
                         n_jobs=n_jobs,
                         verbose=verbose)

    def _fit(self, X, y, cv, **fit_params):
        scores, base_ensemble = self._fit_and_score_ensemble(X, y, cv, **fit_params)
        fitted_models, scores = self._prune_by_cv_score(scores, base_ensemble)

        if len(fitted_models) > self.n_estimators_:
            fitted_models, scores = self._prune_by_correlation(fitted_models, scores, X, y)

        self.fitted_models_ = fitted_models
        self.scores_ = scores

    def _prune_by_cv_score(self, scores, base_ensemble, model_names=None):
        mean_scores = scores.mean(axis=1)
        mean_scores = mean_scores.min() / mean_scores

        idx_good_models = numpy.flatnonzero(mean_scores >= self.min_score)
        if len(idx_good_models) == 0:
            raise ValueError("no base estimator exceeds min_score, try decreasing it")

        fitted_models = self._create_cv_ensemble(base_ensemble, idx_good_models, model_names)

        return fitted_models, mean_scores[idx_good_models]

    def _prune_by_correlation(self, fitted_models, scores, X, y):
        n_models = len(fitted_models)

        out = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
            delayed(_score_regressor)(est, X, y, i)
            for i, est in enumerate(fitted_models))

        error = numpy.empty((X.shape[0], n_models), order="F")
        for i, err in out:
            error[:, i] = err

        final_scores = self._add_diversity_score(scores, error)
        sorted_idx = numpy.argsort(-final_scores, kind="mergesort")

        selected_models = sorted_idx[:self.n_estimators_]

        return fitted_models[selected_models], final_scores

    def _predict_estimators(self, X):
        n_models = len(self.fitted_models_)

        out = Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
            delayed(_predict)(est, X, i)
            for i, est in enumerate(self.fitted_models_))

        predictions = numpy.empty((X.shape[0], n_models), order="F")
        for i, p in out:
            predictions[:, i] = p

        return predictions



from os.path import join, dirname

from nose.plugins.attrib import attr
import numpy
from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal, assert_array_equal

from sklearn.preprocessing import scale

from sksurv.svm.minlip import MinlipSurvivalAnalysis, HingeLossSurvivalSVM
from sksurv.datasets import load_gbsg2
from sksurv.metrics import concordance_index_censored
from sksurv.column import encode_categorical
from sksurv.svm._minlip import create_difference_matrix




class TestMinlipToyExample(TestCase):

    def setUp(self):
        self.x = numpy.array([[1., 1.],
                              [10.2, 15.],
                              [20., 5.],
                              [40, 30],
                              [45, 21],
                              [50, 36]])

        self.y = numpy.empty(self.x.shape[0], dtype=[('status', bool), ('time', float)])
        self.y["status"] = numpy.array([True, True, False, True, False, False])
        self.y["time"] = numpy.arange(1, 7) + 2 ** numpy.arange(1, 7)

    def test_toy_create_difference_matrix_direct_neighbor_without_censoring(self):
        status = numpy.ones(self.y.shape, dtype=bool)
        mat = create_difference_matrix(status.astype(numpy.uint8), self.y["time"], kind="next")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 2] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 4] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of self.y["time"]
        actual_diff = mat.dot(self.y["time"])
        expected_diff = (self.y["time"][1:] - self.y["time"][:-1])

        assert_array_almost_equal(expected_diff, actual_diff)

    def test_toy_create_difference_matrix_direct_neighbor_without_censoring_shuffled(self):
        status = numpy.ones(self.y.shape, dtype=bool)
        order = [3, 2, 5, 0, 1, 4]  # = [ 20.  11.  70.   3.   6.  37.]
        time = self.y["time"][order]
        mat = create_difference_matrix(status.astype(numpy.uint8), time, kind="next")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 3] = -1
        expected[0, 4] = 1
        expected[1, 4] = -1
        expected[1, 1] = 1
        expected[2, 1] = -1
        expected[2, 0] = 1
        expected[3, 0] = -1
        expected[3, 5] = 1
        expected[4, 5] = -1
        expected[4, 2] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of self.y["time"]
        actual_diff = mat.dot(time)
        expected_diff = (self.y["time"][1:] - self.y["time"][:-1])

        assert_array_almost_equal(expected_diff, actual_diff)

    def test_toy_create_difference_matrix_direct_neighbor_with_censoring(self):
        mat = create_difference_matrix(self.y["status"].astype(numpy.uint8), self.y["time"], kind="next")

        expected = numpy.zeros((3, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 3] = -1
        expected[2, 4] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of self.y["time"]
        actual_diff = mat.dot(self.y["time"])
        comparable_pairs = numpy.array([True, True, False, True, False])
        expected_diff = (self.y["time"][1:] - self.y["time"][:-1])[comparable_pairs]

        assert_array_almost_equal(expected_diff, actual_diff)

    def test_toy_create_difference_matrix_nearest_neighbor(self):
        status = numpy.repeat(True, len(self.y))
        mat = create_difference_matrix(status.astype(numpy.uint8), self.y["time"], kind="nearest")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 2] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 4] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

    def test_toy_create_difference_matrix_nearest_neighbor_censored(self):
        mat = create_difference_matrix(self.y["status"].astype(numpy.uint8), self.y["time"], kind="nearest")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 1] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 3] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

    def test_toy_create_difference_matrix_full(self):
        status = numpy.repeat(True, len(self.y))
        mat = create_difference_matrix(status.astype(numpy.uint8), self.y["time"], kind="all")

        expected = numpy.zeros((15, 6), dtype=numpy.int8)
        expected[0, 1] = 1
        expected[0, 0] = -1

        expected[1:3, 2] = 1
        expected[1, 0] = -1
        expected[2, 1] = -1

        expected[3:6, 3] = 1
        expected[3, 0] = -1
        expected[4, 1] = -1
        expected[5, 2] = -1

        expected[6:10, 4] = 1
        expected[6, 0] = -1
        expected[7, 1] = -1
        expected[8, 2] = -1
        expected[9, 3] = -1

        expected[10:15, 5] = 1
        expected[10, 0] = -1
        expected[11, 1] = -1
        expected[12, 2] = -1
        expected[13, 3] = -1
        expected[14, 4] = -1

        assert_array_equal(expected, mat.toarray())

    def test_toy_create_difference_matrix_full_censored(self):
        mat = create_difference_matrix(self.y["status"].astype(numpy.uint8), self.y["time"], kind="all")

        expected = numpy.zeros((11, 6), dtype=numpy.int8)
        expected[0, 1] = 1
        expected[0, 0] = -1

        expected[1:3, 2] = 1
        expected[1, 0] = -1
        expected[2, 1] = -1

        expected[3:5, 3] = 1
        expected[3, 0] = -1
        expected[4, 1] = -1

        expected[5:8, 4] = 1
        expected[5, 0] = -1
        expected[6, 1] = -1
        expected[7, 3] = -1

        expected[8:12, 5] = 1
        expected[8, 0] = -1
        expected[9, 1] = -1
        expected[10, 3] = -1

        assert_array_equal(expected, mat.toarray())

    def test_toy_minlip_fit_cvxopt(self):
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next")
        m.fit(self.x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)
        self.assertEqual(1, m.coef0)
        expected_coef = numpy.array([[-7.18695994e-02, 7.18695994e-02, -7.51880574e-13,
                                      -2.14618562e-01, 2.14618562e-01, 0]])
        assert_array_almost_equal(m.coef_, expected_coef)

    def test_toy_minlip_fit_cvxpy(self):
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=2, pairs="next")
        m.fit(self.x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)
        self.assertEqual(1, m.coef0)
        expected_coef = numpy.array([[-7.18695994e-02, 7.18695994e-02, -7.51880574e-13,
                                      -2.14618562e-01, 2.14618562e-01, 0]])
        assert_array_almost_equal(m.coef_, expected_coef)

    def test_toy_minlip_timeit(self):
        m = MinlipSurvivalAnalysis(alpha=1, pairs="next", timeit=7)
        m.fit(self.x, self.y)

        self.assertEqual(7, len(m.timings_))

    def test_toy_minlip_predict_1_cvxopt(self):
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next")
        m.fit(self.x, self.y)

        p = m.predict(self.x)
        v = concordance_index_censored(self.y['status'], self.y['time'], p)

        self.assertEqual(1.0, v[0])
        self.assertEqual(11, v[1])
        self.assertEqual(0, v[2])
        self.assertEqual(0, v[3])
        self.assertEqual(0, v[4])

    def test_toy_minlip_predict_1_cvxpy(self):
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, pairs="next")
        m.fit(self.x, self.y)

        p = m.predict(self.x)
        v = concordance_index_censored(self.y['status'], self.y['time'], p)

        self.assertEqual(1.0, v[0])
        self.assertEqual(11, v[1])
        self.assertEqual(0, v[2])
        self.assertEqual(0, v[3])
        self.assertEqual(0, v[4])

    def test_toy_minlip_predict_2_cvxopt(self):
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next")
        y = self.y.copy()
        y["time"] = numpy.arange(1, 7)
        m.fit(self.x, y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.34162365, -5.37435297]), p)

    def test_toy_minlip_predict_2_cvxpy(self):
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, pairs="next")
        y = self.y.copy()
        y["time"] = numpy.arange(1, 7)
        m.fit(self.x, y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.341626, -5.374394]), p)

    def test_toy_hinge_fit(self):
        m = HingeLossSurvivalSVM(alpha=1)
        m.fit(self.x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(self.x)
        v = concordance_index_censored(self.y['status'], self.y['time'], p)

        self.assertEqual(1.0, v[0])
        self.assertEqual(11, v[1])
        self.assertEqual(0, v[2])
        self.assertEqual(0, v[3])
        self.assertEqual(0, v[4])

    def test_toy_hinge_predict_cvxopt(self):
        m = HingeLossSurvivalSVM(solver="cvxopt", alpha=1)
        m.fit(self.x, self.y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.341622, -5.374336]), p)

    def test_toy_hinge_predict_cvxpy(self):
        m = HingeLossSurvivalSVM(solver="cvxpy", alpha=1)
        m.fit(self.x, self.y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.34162189, -5.37433203]), p)

    def test_toy_hinge_nearest_fit(self):
        m = HingeLossSurvivalSVM(alpha=1, pairs="nearest")
        m.fit(self.x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(self.x)
        v = concordance_index_censored(self.y['status'], self.y['time'], p)

        self.assertEqual(1.0, v[0])
        self.assertEqual(11, v[1])
        self.assertEqual(0, v[2])
        self.assertEqual(0, v[3])
        self.assertEqual(0, v[4])

    def test_toy_hinge_nearest_predict_cvxopt(self):
        m = HingeLossSurvivalSVM(solver="cvxopt", alpha=1, pairs="nearest")
        m.fit(self.x, self.y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.341623, -5.374339]), p)

    def test_toy_hinge_nearest_predict_cvxpy(self):
        m = HingeLossSurvivalSVM(solver="cvxpy", alpha=1, pairs="nearest")
        m.fit(self.x, self.y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.34161366, -5.37419721]), p)


class TestMinlip(TestCase):

    def setUp(self):
        x, y = load_gbsg2()
        self.x = encode_categorical(x)
        self.y = y

    def test_breast_cancer_cvxopt(self):
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next")
        m.fit(self.x.values, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(self.x.values)
        v = concordance_index_censored(self.y['cens'], self.y['time'], p)

        expected = numpy.array([0.59570007214139709, 79271, 53801, 0, 32])

        assert_array_almost_equal(expected, v)

    def test_breast_cancer_cvxpy(self):
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, pairs="next")
        m.fit(self.x.values, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(self.x.values)
        v = concordance_index_censored(self.y['cens'], self.y['time'], p)
        expected = numpy.array([0.59576770470121443, 79280, 53792, 0, 32])

        assert_array_almost_equal(expected, v)

    def test_breast_cancer_rbf_cvxopt(self):
        x = scale(self.x.values)
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, kernel="rbf",
                                   gamma=32, pairs="next")
        m.fit(x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(x)
        v = concordance_index_censored(self.y['cens'], self.y['time'], p)

        self.assertAlmostEqual(0.64607505711193935, v[0])
        self.assertEqual(85974, v[1])
        self.assertEqual(47097, v[2])
        self.assertEqual(1, v[3])
        self.assertEqual(32, v[4])

    @attr('slow')
    def test_breast_cancer_rbf_cvxpy(self):
        x = scale(self.x.values)
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, kernel="rbf",
                                   gamma=32, pairs="next")
        m.fit(x, self.y)

        self.assertTupleEqual((1, self.x.shape[0]), m.coef_.shape)

        p = m.predict(x)
        v = concordance_index_censored(self.y['cens'], self.y['time'], p)

        self.assertAlmostEqual(0.6402849585186966, v[0], 3)
        self.assertLessEqual(abs(85203 - v[1]), 1)
        self.assertLessEqual(abs(47869 - v[2]), 1)
        self.assertEqual(0, v[3])
        self.assertEqual(32, v[4])

    def test_unknown_solver(self):
        m = MinlipSurvivalAnalysis(solver=None)
        self.assertRaisesRegex(ValueError, "unknown solver: None",
                               m.fit, self.x.values, self.y)

        m.set_params(solver="i don't know")
        self.assertRaisesRegex(ValueError, "unknown solver: i don't know",
                               m.fit, self.x.values, self.y)

        m.set_params(solver=[('why', 'are'), ('you', 'doing this')])
        self.assertRaisesRegex(ValueError, "unknown solver: \[\('why', 'are'\), \('you', 'doing this'\)\]",
                               m.fit, self.x.values, self.y)

    @attr('slow')
    def test_kernel_precomputed(self):
        from sklearn.metrics.pairwise import pairwise_kernels
        from sklearn.utils.metaestimators import _safe_split

        m = MinlipSurvivalAnalysis(kernel="precomputed", solver="cvxpy")
        K = pairwise_kernels(self.x, metric="rbf")

        train_idx = numpy.arange(50, self.x.shape[0])
        test_idx = numpy.arange(50)
        X_fit, y_fit = _safe_split(m, K, self.y, train_idx)
        X_test, y_test = _safe_split(m, K, self.y, test_idx, train_idx)

        m.fit(X_fit, y_fit)

        p = m.predict(X_test)
        v = concordance_index_censored(y_test['cens'], y_test['time'], p)

        expected = numpy.array([0.508748, 378, 365, 0, 0])

        assert_array_almost_equal(expected, v)


if __name__ == '__main__':
    run_module_suite()



from numpy.testing import TestCase, run_module_suite, assert_array_almost_equal, assert_array_equal
import pandas.util.testing as tm
import pandas
import numpy

from sksurv import column

NUMERIC_DATA_FRAME = pandas.DataFrame(numpy.arange(50).reshape(10, 5))

NON_NUMERIC_DATA_FRAME = pandas.DataFrame({'q1': ['no', 'no', 'yes', 'yes', 'no', 'no', None, 'yes', 'no', None],
                                           'q2': ['maybe', 'no', 'yes', 'maybe', 'yes', 'no', None, 'maybe', 'no',
                                                  'yes'],
                                           'q3': [1, 2, 1, 3, 1, 2, numpy.nan, numpy.nan, 3, 2]})

MIXED_DATA_FRAME = pandas.concat((NUMERIC_DATA_FRAME, NON_NUMERIC_DATA_FRAME), axis=1)

NON_NUMERIC_DATA_FRAME['q3'] = NON_NUMERIC_DATA_FRAME['q3'].astype('category')
MIXED_DATA_FRAME['q3'] = MIXED_DATA_FRAME['q3'].astype('category')


class TestColumn(TestCase):
    def test_standardize_numeric(self):
        result = column.standardize(NUMERIC_DATA_FRAME)

        expected = numpy.array([[-1.486301, -1.486301, -1.486301, -1.486301, -1.486301],
                                [-1.156012, -1.156012, -1.156012, -1.156012, -1.156012],
                                [-0.825723, -0.825723, -0.825723, -0.825723, -0.825723],
                                [-0.495434, -0.495434, -0.495434, -0.495434, -0.495434],
                                [-0.165145, -0.165145, -0.165145, -0.165145, -0.165145],
                                [0.165145, 0.165145, 0.165145, 0.165145, 0.165145],
                                [0.495434, 0.495434, 0.495434, 0.495434, 0.495434],
                                [0.825723, 0.825723, 0.825723, 0.825723, 0.825723],
                                [1.156012, 1.156012, 1.156012, 1.156012, 1.156012],
                                [1.486301, 1.486301, 1.486301, 1.486301, 1.486301]])

        self.assertTrue(isinstance(result, pandas.DataFrame))
        assert_array_almost_equal(expected, result)

    def test_standardize_non_numeric(self):
        result = column.standardize(NON_NUMERIC_DATA_FRAME)

        self.assertTrue(isinstance(result, pandas.DataFrame))
        tm.assert_frame_equal(NON_NUMERIC_DATA_FRAME, result)

    def test_standardize_mixed(self):
        result = column.standardize(MIXED_DATA_FRAME)

        expected = numpy.array([[-1.486301, -1.486301, -1.486301, -1.486301, -1.486301],
                                [-1.156012, -1.156012, -1.156012, -1.156012, -1.156012],
                                [-0.825723, -0.825723, -0.825723, -0.825723, -0.825723],
                                [-0.495434, -0.495434, -0.495434, -0.495434, -0.495434],
                                [-0.165145, -0.165145, -0.165145, -0.165145, -0.165145],
                                [0.165145, 0.165145, 0.165145, 0.165145, 0.165145],
                                [0.495434, 0.495434, 0.495434, 0.495434, 0.495434],
                                [0.825723, 0.825723, 0.825723, 0.825723, 0.825723],
                                [1.156012, 1.156012, 1.156012, 1.156012, 1.156012],
                                [1.486301, 1.486301, 1.486301, 1.486301, 1.486301]])

        self.assertTrue(isinstance(result, pandas.DataFrame))
        assert_array_almost_equal(expected, result.iloc[:, :NUMERIC_DATA_FRAME.shape[1]].values)

        tm.assert_frame_equal(NON_NUMERIC_DATA_FRAME, result.iloc[:, NUMERIC_DATA_FRAME.shape[1]:])

    def test_standardize_numpy_array(self):
        result = column.standardize(MIXED_DATA_FRAME.values)

        expected = numpy.array([[-1.486301, -1.486301, -1.486301, -1.486301, -1.486301],
                                [-1.156012, -1.156012, -1.156012, -1.156012, -1.156012],
                                [-0.825723, -0.825723, -0.825723, -0.825723, -0.825723],
                                [-0.495434, -0.495434, -0.495434, -0.495434, -0.495434],
                                [-0.165145, -0.165145, -0.165145, -0.165145, -0.165145],
                                [0.165145, 0.165145, 0.165145, 0.165145, 0.165145],
                                [0.495434, 0.495434, 0.495434, 0.495434, 0.495434],
                                [0.825723, 0.825723, 0.825723, 0.825723, 0.825723],
                                [1.156012, 1.156012, 1.156012, 1.156012, 1.156012],
                                [1.486301, 1.486301, 1.486301, 1.486301, 1.486301]])

        self.assertTrue(isinstance(result, numpy.ndarray))
        assert_array_almost_equal(expected, result[:, :NUMERIC_DATA_FRAME.shape[1]])

        assert_array_equal(pandas.isnull(NON_NUMERIC_DATA_FRAME),
                           pandas.isnull(result[:, NUMERIC_DATA_FRAME.shape[1]:]))

        non_nan_idx = [0, 1, 2, 3, 4, 5, 8, 9]

        assert_array_equal(NON_NUMERIC_DATA_FRAME.iloc[non_nan_idx, :].values,
                           result[:, NUMERIC_DATA_FRAME.shape[1]:][non_nan_idx, :])


class TestEncodeCategorical(TestCase):
    def test_series_categorical(self):
        input_series = pandas.Series(pandas.Categorical.from_codes([1, 1, 0, 2, 0, 1, 2, 1, 2, 0, 0, 1, 2, 2],
                                                                   ["small", "medium", "large"], ordered=False),
                                     name="a_series")
        expected_df = pandas.DataFrame.from_items(
            [("a_series=medium", numpy.array([1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0], dtype=float)),
             ("a_series=large", numpy.array([0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1], dtype=float))
            ])

        actual_df = column.encode_categorical(input_series)

        tm.assert_frame_equal(actual_df, expected_df, check_exact=True)

    def test_series_numeric(self):
        input_series = pandas.Series([0.5, 0.1, 10, 25, 3.8, 11, 2256, -1, -0.2, 3.14], name="a_series")

        self.assertRaisesRegex(TypeError, "series must be of categorical dtype, but was float",
                               column.encode_categorical, input_series)

    def test_case1(self):
        a = numpy.concatenate((
            numpy.repeat(["large"], 10),
            numpy.repeat(["small"], 5),
            numpy.repeat(["tiny"], 13),
            numpy.repeat(["medium"], 3)))
        b = numpy.concatenate((
            numpy.repeat(["yes"], 8),
            numpy.repeat(["no"], 23)))

        rnd = numpy.random.RandomState(0)
        c = rnd.randn(len(a))

        df = pandas.DataFrame({"a_category": a,
                               "a_binary": b,
                               "a_number": c.copy()})

        actual_df = column.encode_categorical(df)

        eb = numpy.concatenate((
            numpy.repeat([1.], 8),
            numpy.repeat([0.], 23)))

        a_tiny = numpy.zeros(31, dtype=float)
        a_tiny[15:28] = 1

        a_small = numpy.zeros(31, dtype=float)
        a_small[10:15] = 1

        a_medium = numpy.zeros(31, dtype=float)
        a_medium[-3:] = 1

        expected_df = pandas.DataFrame({"a_number": c.copy(),
                                        "a_binary=yes": eb,
                                        "a_category=medium": a_medium,
                                        "a_category=small": a_small,
                                        "a_category=tiny": a_tiny})

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df, expected_df, check_exact=True)

    def test_duplicate_index(self):
        a = numpy.concatenate((
            numpy.repeat(["large"], 10),
            numpy.repeat(["small"], 6),
            numpy.repeat(["tiny"], 13),
            numpy.repeat(["medium"], 3)))
        rnd = numpy.random.RandomState(0)
        c = rnd.randn(len(a))

        index = numpy.ceil(numpy.arange(0, len(a) // 2, 0.5))
        df = pandas.DataFrame.from_items([("a_category", pandas.Series(a, index=index)),
                                          ("a_number", pandas.Series(c, index=index, copy=True))])

        actual_df = column.encode_categorical(df)

        expected_df = pandas.DataFrame(numpy.zeros((32, 3), dtype=numpy.float_),
                                       index=index,
                                       columns=["a_category=medium", "a_category=small", "a_category=tiny"])
        # tiny
        expected_df.iloc[16:29, 2] = 1
        # small
        expected_df.iloc[10:16, 1] = 1
        # medium
        expected_df.iloc[-3:, 0] = 1

        expected_df["a_number"] = c

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df, expected_df, check_exact=True)

    def test_case_numeric(self):
        a = numpy.array([0, 1, 1, 0, 1, 0, 0, 1, 0, 1], dtype=object)
        b = numpy.array([1, 2, 1, 3, 2, 1, 3, 2, 3, 1], dtype=object)
        c = numpy.array([1./128, 1./32, 1., 1./8, 1./32, 1., 1./128, 1./8, 1., 1./32], dtype=object)

        df = pandas.DataFrame({"a_binary_int": a.copy(),
                               "a_three_int": b.copy(),
                               "a_four_float": c.copy()})

        actual_df = column.encode_categorical(df)

        expected_df = pandas.DataFrame({
            "a_binary_int=1": a.astype(float),
            "a_three_int=2": (b == 2).astype(float),
            "a_three_int=3": (b == 3).astype(float),
            "a_four_float={}".format(1. / 32): (c == 1. / 32).astype(float),
            "a_four_float={}".format(1. / 8): (c == 1. / 8).astype(float),
            "a_four_float={}".format(1.): (c == 1.).astype(float),
        })

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df, expected_df, check_exact=True)

    def test_with_missing(self):
        b = numpy.concatenate((
            numpy.repeat(["yes"], 5),
            numpy.repeat([None], 10),
            numpy.repeat(["no"], 16)))

        rnd = numpy.random.RandomState(0)
        c = rnd.randn(len(b))

        df = pandas.DataFrame({"a_binary": b,
                               "a_number": c.copy()})

        actual_df = column.encode_categorical(df)

        eb = numpy.concatenate((
            numpy.repeat([1.], 5),
            numpy.repeat([numpy.nan], 10),
            numpy.repeat([0.], 16)))

        expected_df = pandas.DataFrame({"a_number": c.copy(),
                                        "a_binary=yes": eb})

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df.isnull(), expected_df.isnull())
        tm.assert_frame_equal(actual_df.dropna(), expected_df.dropna(), check_exact=True)

    def test_drop_all_missing(self):
        b = numpy.concatenate((
            numpy.repeat(["yes"], 5),
            numpy.repeat([None], 10),
            numpy.repeat(["no"], 16)))

        all_missing = numpy.repeat([None], len(b))

        df = pandas.DataFrame({"a_binary": b,
                               "bogus": all_missing})

        actual_df = column.encode_categorical(df)

        eb = numpy.concatenate((
            numpy.repeat([1.], 5),
            numpy.repeat([numpy.nan], 10),
            numpy.repeat([0.], 16)))

        expected_df = pandas.DataFrame({"a_binary=yes": eb})

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df.isnull(), expected_df.isnull())
        tm.assert_frame_equal(actual_df.dropna(), expected_df.dropna(), check_exact=True)

    def test_retain_all_missing(self):
        b = numpy.concatenate((
            numpy.repeat(["yes"], 5),
            numpy.repeat([None], 10),
            numpy.repeat(["no"], 16)))

        all_missing = numpy.repeat([None], len(b))

        df = pandas.DataFrame({"a_binary": b,
                               "bogus": all_missing})

        actual_df = column.encode_categorical(df, allow_drop=False)

        eb = numpy.concatenate((
            numpy.repeat([1.], 5),
            numpy.repeat([numpy.nan], 10),
            numpy.repeat([0.], 16)))

        expected_df = pandas.DataFrame({"a_binary=yes": eb,
                                        "bogus": all_missing.copy()})

        self.assertTupleEqual(actual_df.shape, expected_df.shape)
        tm.assert_frame_equal(actual_df.isnull(), expected_df.isnull())
        tm.assert_frame_equal(actual_df.dropna(), expected_df.dropna(), check_exact=True)


class TestCategoricalToNumeric(TestCase):

    def test_series(self):
        input_series = pandas.Series(["a", "a", "b", "b", "b", "c"], name="Thr33",
                               index=["Alpha", "Beta", "Gamma", "Delta", "Eta", "Mu"])
        expected = pandas.Series([0, 0, 1, 1, 1, 2], name="Thr33",
                                 index=["Alpha", "Beta", "Gamma", "Delta", "Eta", "Mu"],
                                 dtype=numpy.int64)

        actual = column.categorical_to_numeric(input_series)

        tm.assert_series_equal(actual, expected, check_exact=True)

    def test_bool_series(self):
        input_series = pandas.Series([True, True, False, False, True, False, True], name="human",
                                     index=["Alpha", "Beta", "Gamma", "Delta", "Eta", "Mu", "Zeta"])
        expected = pandas.Series([1, 1, 0, 0, 1, 0, 1], name="human",
                                 index=["Alpha", "Beta", "Gamma", "Delta", "Eta", "Mu", "Zeta"],
                                 dtype=numpy.int64)

        actual = column.categorical_to_numeric(input_series)

        tm.assert_series_equal(actual, expected, check_exact=True)

    def test_data_frame(self):
        a = numpy.concatenate((
            numpy.repeat(["large"], 10),
            numpy.repeat(["small"], 5),
            numpy.repeat(["tiny"], 13),
            numpy.repeat(["medium"], 3)))
        b = numpy.concatenate((
            numpy.repeat(["yes"], 8),
            numpy.repeat(["no"], 23)))

        rnd = numpy.random.RandomState(0)
        c = rnd.randn(len(a))

        input_df = pandas.DataFrame({"a_category": a,
                                     "a_binary": b,
                                     "a_number": c.copy()})

        a_num = numpy.concatenate((
            numpy.repeat([0], 10),
            numpy.repeat([2], 5),
            numpy.repeat([3], 13),
            numpy.repeat([1], 3))).astype(numpy.int64)
        b_num = numpy.concatenate((
            numpy.repeat([1], 8),
            numpy.repeat([0], 23))).astype(numpy.int64)
        expected = pandas.DataFrame({"a_category": a_num,
                                     "a_binary": b_num,
                                     "a_number": c.copy()})

        actual = column.categorical_to_numeric(input_df)

        tm.assert_frame_equal(actual, expected, check_exact=True)


if __name__ == '__main__':
    run_module_suite()



import os.path

import numpy
from numpy.testing import TestCase, run_module_suite

from sksurv.metrics import concordance_index_censored

WHAS500_DATA_FILE = os.path.join(os.path.dirname(__file__), 'data', 'whas500_predictions.csv')


class TestConcordanceIndex(TestCase):
    def test_concordance_index_no_censoring_all_correct(self):
        time = [1, 5, 6, 11, 34, 45, 46, 50]
        event = numpy.repeat(True, len(time))
        estimate = numpy.arange(len(time))[::-1]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)
        self.assertEqual(28, con)
        self.assertEqual(0, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(0, tie_t)
        self.assertEqual(1.0, c)

    def test_concordance_index_no_censoring_all_wrong(self):
        time = [1, 5, 6, 11, 34, 45, 46, 50]
        event = numpy.repeat(True, len(time))
        # order is exactly reversed
        estimate = numpy.arange(len(time))

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)
        self.assertEqual(0, con)
        self.assertEqual(28, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(0, tie_t)
        self.assertEqual(0.0, c)

    def test_concordance_index_no_ties(self):
        event = [False, True, True, False, False, True, False, False]
        time = [1, 5, 6, 11, 34, 45, 46, 50]
        estimate = [5, 8, 11, 34, 12, 3, 9, 12]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)

        self.assertEqual(3, con)
        self.assertEqual(10, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(0, tie_t)
        self.assertAlmostEqual(0.2307692, c, 6)

    def test_concordance_index_with_tied_time(self):
        event = [False, True, True, False, True, False, True, False, False]
        time = [1, 5, 6, 11, 11, 34, 45, 45, 50]
        estimate = [5, 8, 11, 19, 34, 12, 3, 9, 12]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)

        self.assertEqual(8, con)
        self.assertEqual(12, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(0, tie_t)
        self.assertAlmostEqual(0.4, c, 6)

    def test_concordance_index_with_tied_time2(self):
        event = [False, True, True, False, False, False, True, False, False]
        time = [1, 5, 6, 11, 11, 34, 45, 45, 50]
        estimate = [5, 8, 11, 19, 34, 12, 3, 9, 12]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)

        self.assertEqual(3, con)
        self.assertEqual(12, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(0, tie_t)
        self.assertAlmostEqual(0.2, c, 6)

    def test_concordance_index_with_tied_event(self):
        event = [False, True, False, True, True, False, True, False, False]
        time = [1, 5, 6, 11, 11, 34, 45, 45, 50]
        estimate = [5, 8, 11, 19, 34, 12, 3, 9, 12]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event[::-1], time[::-1], estimate[::-1])

        self.assertEqual(9, con)
        self.assertEqual(8, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(1, tie_t)
        self.assertAlmostEqual(0.5294118, c, 6)

    def test_concordance_index_with_tied_event_and_time(self):
        event = [True, False, False, False, True, False, True, True, False, False, False, True]
        time = [34, 11, 11, 5, 1, 89, 13, 45, 7, 13, 9, 13]
        estimate = [1, 19, 13, 13, 15, 14, 19, 23, 11, 10, 11, 1]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, estimate)

        self.assertEqual(12, con)
        self.assertEqual(9, dis)
        self.assertEqual(1, tie_r)
        self.assertEqual(1, tie_t)
        self.assertAlmostEqual(0.5681818, c, 6)

    def test_concordance_index(self):
        dat = numpy.loadtxt(WHAS500_DATA_FILE, delimiter=",")
        event = dat[:, 0] == 1
        time = dat[:, 1]
        risk = dat[:, 2]

        c, con, dis, tie_r, tie_t = concordance_index_censored(event, time, risk)
        self.assertEqual(57849, con)
        self.assertEqual(17300, dis)
        self.assertEqual(0, tie_r)
        self.assertEqual(119, tie_t)
        self.assertAlmostEqual(0.7697907, c, 6)

    def test_different_length(self):
        event = numpy.array([True, False, False, True, True, False])
        time = numpy.array([1, 5, 10, 12, 7, 65])
        estimate = numpy.array([12, 8, 1, 89, 56, 13])

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event, time[:3], estimate)

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event, time, estimate[:3])

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event[:3], time, estimate, )

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event, time[:3], estimate[:3])

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event[:3], time, estimate[:3])

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: .+",
                               concordance_index_censored, event[:3], time[:3], estimate)

    def test_boolean_event(self):
        event = numpy.array([1, 0, 0, 1, 1, 0])
        time = numpy.array([1, 5, 10, 12, 7, 65])
        estimate = numpy.array([12, 8, 1, 89, 56, 13])

        self.assertRaisesRegex(ValueError, "only boolean arrays are supported as class labels for survival analysis.+",
                               concordance_index_censored, event, time, estimate)

    def test_min_samples(self):
        event = numpy.array([False])
        time = numpy.array([10])
        estimate = numpy.array([12])

        self.assertRaisesRegex(ValueError, "Need a minimum of two samples",
                               concordance_index_censored, event, time, estimate)

    def test_all_censored(self):
        event = numpy.array([False, False])
        time = numpy.array([10, 12])
        estimate = numpy.array([12, 13])

        self.assertRaisesRegex(ValueError, "All samples are censored",
                               concordance_index_censored, event, time, estimate)

    def test_all_finite(self):
        event = numpy.array([True, False, None, True, True, False])
        time = numpy.array([1, 5, 10, 12, 7, 65], dtype=float)
        estimate = numpy.array([12, 8, 1, 89, 56, 13], dtype=float)

        self.assertRaisesRegex(ValueError, "Input contains NaN, infinity or a value too large for .+",
                               concordance_index_censored, event, time, estimate)

        event[2] = False
        time[3] = numpy.nan
        self.assertRaisesRegex(ValueError, "Input contains NaN, infinity or a value too large for .+",
                               concordance_index_censored, event, time, estimate)

        time[3] = numpy.nan
        estimate[5] = numpy.inf
        self.assertRaisesRegex(ValueError, "Input contains NaN, infinity or a value too large for .+",
                               concordance_index_censored, event, time, estimate)


if __name__ == '__main__':
    run_module_suite()



import os
import os.path
import sys

# NumPy 1.11.2 contains a bug which prevents submodules from working correctly
# on Python 3.4 unless importlib.machinery has been imported at some time.
try:
    import importlib.machinery
except:
    pass

from setuptools import find_packages


with open('README.rst') as fp:
    long_description = fp.read()


def configuration(parent_package='', top_path=None):
    if os.path.exists('MANIFEST'):
        os.remove('MANIFEST')

    from numpy.distutils.misc_util import Configuration

    config = Configuration(None, parent_package, top_path)

    # Avoid non-useful msg:
    # "Ignoring attempt to set 'name' (from ... "
    config.set_options(ignore_setup_xxx_py=True,
                       assume_default_configuration=True,
                       delegate_options_to_subpackages=True,
                       quiet=True)

    config.add_subpackage('sksurv')

    return config


def setup_package():
    metadata = dict(name='scikit-survival',
                    url='https://github.com/sebp/scikit-survival',
                    author='Sebastian Pölsterl',
                    author_email='sebp@k-d-w.org',
                    description='Survival analysis built on top of scikit-learn',
                    long_description=long_description,
                    license="GPLv3+",
                    packages=find_packages(),
                    classifiers=['Development Status :: 4 - Beta',
                                 'Intended Audience :: Science/Research',
                                 'Intended Audience :: Developers',
                                 'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
                                 'Operating System :: POSIX',
                                 'Operating System :: Unix',
                                 'Programming Language :: C',
                                 'Programming Language :: Python',
                                 'Programming Language :: Python :: 3.4',
                                 'Programming Language :: Python :: 3.5',
                                 'Programming Language :: Python :: 3.6',
                                 'Topic :: Software Development',
                                 'Topic :: Scientific/Engineering',
                    ],
                    use_scm_version=True,
                    setup_requires=['setuptools_scm'],
                    install_requires=[
                        'cvxopt',
                        'cvxpy',
                        'numexpr',
                        'numpy',
                        'pandas >=0.18, <0.21',
                        'scipy',
                        'scikit-learn >=0.18.0, <0.19'],
                    extras_require={
                        'tests': [
                            'nose',
                            'coverage'],
                        'docs': [
                            'sphinx >= 1.4',
                            'numpydoc']}
    )

    if (len(sys.argv) >= 2
        and ('--help' in sys.argv[1:] or sys.argv[1]
        in ('--help-commands', 'egg_info', '--version', 'clean'))):

        # For these actions, NumPy is not required.
        from setuptools import setup
    else:
        from numpy.distutils.core import setup

        metadata['configuration'] = configuration

    setup(**metadata)


if __name__ == "__main__":
    py_version = sys.version_info[:2]
    if py_version < (3, 4):
        raise RuntimeError('Python 3.4 or later is required')

    setup_package()



from pkg_resources import resource_filename
import warnings

import numpy
import pandas

from ..column import standardize, categorical_to_numeric
from ..io import loadarff
from ..util import safe_concat

__all__ = ["get_x_y",
           "load_arff_files_standardized",
           "load_aids",
           "load_breast_cancer",
           "load_gbsg2",
           "load_whas500",
           "load_veterans_lung_cancer"]


def _get_x_y_survival(dataset, col_event, col_time, val_outcome):
    if col_event is None or col_time is None:
        y = None
        x_frame = dataset
    else:
        y = numpy.empty(dtype=[(col_event, numpy.bool), (col_time, numpy.float64)],
                        shape=dataset.shape[0])
        y[col_event] = (dataset[col_event] == val_outcome).values
        y[col_time] = dataset[col_time].values

        x_frame = dataset.drop([col_event, col_time], axis=1)

    return x_frame, y


def _get_x_y_other(dataset, col_label):
    if col_label is None:
        y = None
        x_frame = dataset
    else:
        y = dataset.loc[:, col_label]
        x_frame = dataset.drop(col_label, axis=1)

    return x_frame, y


def get_x_y(data_frame, attr_labels, pos_label=None, survival=True):
    """Split data frame into features and labels.

    Parameters
    ----------
    data_frame : pandas.DataFrame, shape = [n_samples, n_columns]
        A data frame.

    attr_labels : sequence of str or None
        A list of one or more columns that are considered the label.
        If `survival` is `True`, then attr_labels has two elements:
        1) the name of the column denoting the event indicator, and
        2) the name of the column denoting the survival time.
        If the sequence contains `None`, then labels are not retrieved
        and only a data frame with features is returned.

    pos_label : any, optional
        Which value of the event indicator column denotes that a
        patient experienced an event. This value is ignored if
        `survival` is `False`.

    survival : bool, optional, default: True
        Whether to return `y` that can be used for survival analysis.

    Returns
    -------
    X : pandas.DataFrame, shape = [n_samples, n_columns - len(attr_labels)]
        Data frame containing features.

    y : None or pandas.DataFrame, shape = [n_samples, len(attr_labels])
        Data frame containing columns with supervised information.
        If `survival` was `True`, then the column denoting the event
        indicator will be boolean and survival times will be float.
        If `attr_labels` contains `None`, y is set to `None`.
    """
    if survival:
        if len(attr_labels) != 2:
            raise ValueError("expected sequence of length two for attr_labels, but got %d" % len(attr_labels))
        if pos_label is None:
            raise ValueError("pos_label needs to be specified if survival=True")
        return _get_x_y_survival(data_frame, attr_labels[0], attr_labels[1], pos_label)

    return _get_x_y_other(data_frame, attr_labels)


def load_arff_files_standardized(path_training, attr_labels, pos_label=None, path_testing=None, survival=True,
                                 standardize_numeric=True, to_numeric=True):
    """Load dataset in ARFF format.

    Parameters
    ----------
    path_training : str
        Path to ARFF file containing data.

    attr_labels : sequence of str
        Names of attributes denoting dependent variables.
        If ``survival`` is set, it must be a sequence with two items:
        the name of the event indicator and the name of the survival/censoring time.

    pos_label : any type, optional
        Value corresponding to an event in survival analysis.
        Only considered if ``survival`` is ``True``.

    path_testing : str, optional
        Path to ARFF file containing hold-out data. Only columns that are available in both
        training and testing are considered (excluding dependent variables).
        If ``standardize_numeric`` is set, data is normalized by considering both training
        and testing data.

    survival : bool, optional, default=True
        Whether the dependent variables denote event indicator and survival/censoring time.

    standardize_numeric : bool, optional, default=True
        Whether to standardize data to zero mean and unit variance.
        See :func:`sksurv.column.standardize`.

    to_numeric : boo, optional, default=True
        Whether to convert categorical variables to numeric values.
        See :func:`sksurv.column.categorical_to_numeric`.

    Returns
    -------
    x_train : pandas.DataFrame, shape = [n_train, n_features]
        Training data.

    y_train : pandas.DataFrame, shape = [n_train, n_labels]
        Dependent variables of training data.

    x_test : None or pandas.DataFrame, shape = [n_train, n_features]
        Testing data if `path_testing` was provided.

    y_test : None or pandas.DataFrame, shape = [n_train, n_labels]
        Dependent variables of testing data if `path_testing` was provided.
    """
    dataset = loadarff(path_training)
    if "index" in dataset.columns:
        dataset.index = dataset["index"].astype(object)
        dataset.drop("index", axis=1, inplace=True)

    x_train, y_train = get_x_y(dataset, attr_labels, pos_label, survival)

    if path_testing is not None:
        test_dataset = loadarff(path_testing)
        if "index" in test_dataset.columns:
            test_dataset.index = test_dataset["index"].astype(object)
            test_dataset.drop("index", axis=1, inplace=True)

        has_labels = pandas.Index(attr_labels).isin(test_dataset.columns).all()
        if not has_labels:
            if survival:
                attr_labels = [None, None]
            else:
                attr_labels = None
        x_test, y_test = get_x_y(test_dataset, attr_labels, pos_label, survival)

        if len(x_train.columns.symmetric_difference(x_test.columns)) > 0:
            warnings.warn("Restricting columns to intersection between training and testing data",
                          stacklevel=2)

            cols = x_train.columns.intersection(x_test.columns)
            if len(cols) == 0:
                raise ValueError("columns of training and test data do not intersect")

            x_train = x_train.loc[:, cols]
            x_test = x_test.loc[:, cols]

        x = safe_concat((x_train, x_test), axis=0)
        if standardize_numeric:
            x = standardize(x)
        if to_numeric:
            x = categorical_to_numeric(x)

        n_train = x_train.shape[0]
        x_train = x.iloc[:n_train, :]
        x_test = x.iloc[n_train:, :]
    else:
        if standardize_numeric:
            x_train = standardize(x_train)
        if to_numeric:
            x_train = categorical_to_numeric(x_train)

        x_test = None
        y_test = None

    return x_train, y_train, x_test, y_test


def load_whas500():
    """Load and return the Worcester Heart Attack Study dataset

    The dataset has 500 samples and 14 features.
    The endpoint is death, which occurred for 215 patients (43.0%).

    Returns
    -------
    x : pandas.DataFrame
        The measurements for each patient.

    y : structured array with 2 fields
        *fstat*: boolean indicating whether the endpoint has been reached
        or the event time is right censored.

        *lenfol*: total length of follow-up (days from hospital admission date
        to date of last follow-up)

    References
    ----------
    .. [1] http://www.umass.edu/statdata/statdata/data/

    .. [2] Hosmer, D., Lemeshow, S., May, S.:
        "Applied Survival Analysis: Regression Modeling of Time to Event Data."
        John Wiley & Sons, Inc. (2008)
    """
    fn = resource_filename(__name__, 'data/whas500.arff')
    return get_x_y(loadarff(fn), attr_labels=['fstat', 'lenfol'], pos_label='1')


def load_gbsg2():
    """Load and return the German Breast Cancer Study Group 2 dataset

    The dataset has 686 samples and 8 features.
    The endpoint is recurrence free survival, which occurred for 299 patients (43.6%).

    Returns
    -------
    x : pandas.DataFrame
        The measurements for each patient.

    y : structured array with 2 fields
        *cens*: boolean indicating whether the endpoint has been reached
        or the event time is right censored.

        *time*: total length of follow-up

    References
    ----------
    .. [1] http://ascopubs.org/doi/abs/10.1200/jco.1994.12.10.2086

    .. [2] Schumacher, M., Basert, G., Bojar, H., et al.
        "Randomized 2 × 2 trial evaluating hormonal treatment and the duration of chemotherapy
        in node-positive breast cancer patients."
        Journal of Clinical Oncology 12, 2086–2093. (1994)
    """
    fn = resource_filename(__name__, 'data/GBSG2.arff')
    return get_x_y(loadarff(fn), attr_labels=['cens', 'time'], pos_label='1')


def load_veterans_lung_cancer():
    """Load and return the Worcester Heart Attack Study dataset

    The dataset has 137 samples and 6 features.
    The endpoint is death, which occurred for 128 patients (93.4%).

    Returns
    -------
    x : pandas.DataFrame
        The measurements for each patient.

    y : structured array with 2 fields
        *Status*: boolean indicating whether the endpoint has been reached
        or the event time is right censored.

        *Survival_in_days*: total length of follow-up

    References
    ----------
    .. [1] Kalbfleisch, J.D., Prentice, R.L.:
        "The Statistical Analysis of Failure Time Data." John Wiley & Sons, Inc. (2002)
    """
    fn = resource_filename(__name__, 'data/veteran.arff')
    return get_x_y(loadarff(fn), attr_labels=['Status', 'Survival_in_days'], pos_label="'dead'")


def load_aids(endpoint="aids"):
    """Load and return the AIDS Clinical Trial dataset

    The dataset has 1,151 samples and 11 features.
    The dataset has 2 endpoints:

    1. AIDS defining event, which occurred for 96 patients (8.3%)
    2. Death, which occurred for 26 patients (2.3%)

    Parameters
    ----------
    endpoint : aids|death
        The endpoint

    Returns
    -------
    x : pandas.DataFrame
        The measurements for each patient.

    y : structured array with 2 fields
        *censor*: boolean indicating whether the endpoint has been reached
        or the event time is right censored.

        *time*: total length of follow-up

        If ``endpoint`` is death, the fields are named *censor_d* and *time_d*.

    References
    ----------
    .. [1] http://www.umass.edu/statdata/statdata/data/

    .. [2] Hosmer, D., Lemeshow, S., May, S.:
        "Applied Survival Analysis: Regression Modeling of Time to Event Data."
        John Wiley & Sons, Inc. (2008)
    """
    labels_aids = ['censor', 'time']
    labels_death = ['censor_d', 'time_d']
    if endpoint == "aids":
        attr_labels = labels_aids
        drop_columns = labels_death
    elif endpoint == "death":
        attr_labels = labels_death
        drop_columns = labels_aids
    else:
        raise ValueError("endpoint must be 'aids' or 'death'")

    fn = resource_filename(__name__, 'data/actg320.arff')
    x, y = get_x_y(loadarff(fn), attr_labels=attr_labels, pos_label='1')
    x.drop(drop_columns, axis=1, inplace=True)
    return x, y


def load_breast_cancer():
    """Load and return the breast cancer dataset

    The dataset has 198 samples and 80 features.
    The endpoint is the presence of distance metastases, which occurred for 51 patients (25.8%).

    Returns
    -------
    x : pandas.DataFrame
        The measurements for each patient.

    y : structured array with 2 fields
        *e.tdm*: boolean indicating whether the endpoint has been reached
        or the event time is right censored.

        *t.tdm*: time to distant metastasis (days)

    References
    ----------
    .. [1] https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE7390

    .. [2] Desmedt, C., Piette, F., Loi et al.:
        "Strong Time Dependence of the 76-Gene Prognostic Signature for Node-Negative Breast Cancer
        Patients in the TRANSBIG Multicenter Independent Validation Series."
        Clin. Cancer Res. 13(11), 3207–14 (2007)
    """
    fn = resource_filename(__name__, 'data/breast_cancer_GSE7390-metastasis.arff')
    return get_x_y(loadarff(fn), attr_labels=['e.tdm', 't.tdm'], pos_label="'1'")



from os.path import join, dirname

from nose.plugins.attrib import attr
import numpy
from numpy.testing import TestCase, run_module_suite, assert_array_equal, assert_array_almost_equal
import pandas
from scipy.sparse import coo_matrix
from sklearn.metrics import mean_squared_error

from sksurv.datasets import load_whas500
from sksurv.metrics import concordance_index_censored
from sksurv import column
from sksurv.ensemble import ComponentwiseGradientBoostingSurvivalAnalysis, GradientBoostingSurvivalAnalysis


def early_stopping_monitor(i, est, locals_):
    """Returns True on the 10th iteration. """
    return i == 9


class TestGradientBoosting(TestCase):
    def setUp(self):
        x, self.y = load_whas500()

        x = column.categorical_to_numeric(column.standardize(x, with_std=False))
        self.x = x.values
        self.columns = x.columns.tolist()

    def test_fit(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=100, max_depth=3, min_samples_split=10,
                                                 random_state=0)
        model.fit(self.x, self.y)

        self.assertEquals(model.max_features_, 14)
        self.assertFalse(hasattr(model, "oob_improvement_"))

        p = model.predict(self.x)

        expected_cindex = numpy.array([0.86272605091218779, 64826, 10309, 14, 119])
        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        assert_array_almost_equal(expected_cindex, numpy.array(result))

        self.assertTupleEqual((100,), model.train_score_.shape)

        self.assertRaisesRegex(ValueError, "Number of features of the model must match the input. "
                                           "Model n_features is 14 and input n_features is 2 ",
                               model.predict, self.x[:, :2])

    def test_fit_subsample(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=100, max_features=8, subsample=0.6,
                                                 random_state=0)
        model.fit(self.x, self.y)

        self.assertEquals(model.max_features_, 8)
        self.assertTrue(hasattr(model, "oob_improvement_"))

        p = model.predict(self.x)

        expected_cindex = numpy.array([0.8610760, 64709, 10440, 0, 119])
        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        assert_array_almost_equal(expected_cindex, numpy.array(result))

        self.assertTupleEqual((100,), model.train_score_.shape)
        self.assertTupleEqual((100,), model.oob_improvement_.shape)

        self.assertRaisesRegex(ValueError, "Number of features of the model must match the input. "
                                           "Model n_features is 14 and input n_features is 2 ",
                               model.predict, self.x[:, :2])

    @attr('slow')
    def test_fit_dropout(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=100, max_features=8,
                                                 learning_rate=1.0, dropout_rate=0.03,
                                                 random_state=0)
        model.fit(self.x, self.y)

        self.assertFalse(hasattr(model, "oob_improvement_"))
        self.assertEquals(model.max_features_, 8)

        p = model.predict(self.x)

        expected_cindex = numpy.array([0.9094333, 68343, 6806, 0, 119])
        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        assert_array_almost_equal(expected_cindex, numpy.array(result))

    def test_fit_int_param_as_float(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=100.0, max_depth=3.0, min_samples_split=10.0,
                                                 random_state=0)
        params = model.get_params()
        self.assertEqual(100, params["n_estimators"])
        self.assertEqual(3, params["max_depth"])
        self.assertEqual(10, params["min_samples_split"])

        model.set_params(max_leaf_nodes=15.0)
        self.assertEqual(15, model.get_params()["max_leaf_nodes"])

        model.fit(self.x, self.y)
        p = model.predict(self.x)

        expected_cindex = numpy.array([0.90256690042449006, 67826, 7321, 2, 119])
        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        assert_array_almost_equal(expected_cindex, numpy.array(result))

    def test_max_features(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=10, max_features="auto", max_depth=3, random_state=0)
        model.fit(self.x, self.y)

        self.assertEqual(model.max_features_, self.x.shape[1])

        model.set_params(max_features="sqrt")
        model.fit(self.x, self.y)
        self.assertAlmostEqual(model.max_features_, int(numpy.sqrt(self.x.shape[1])))

        model.set_params(max_features="log2")
        model.fit(self.x, self.y)
        self.assertAlmostEqual(model.max_features_, int(numpy.log2(self.x.shape[1])))

        model.set_params(max_features=0.25)
        model.fit(self.x, self.y)
        self.assertAlmostEqual(model.max_features_, int(0.25 * self.x.shape[1]))

        model.set_params(max_features=5)
        model.fit(self.x, self.y)
        self.assertAlmostEqual(model.max_features_, 5)

        model.set_params(max_features=-1)
        self.assertRaisesRegex(ValueError,
                               "max_features must be in \(0, n_features\]",
                               model.fit, self.x, self.y)

        model.set_params(max_features=-1.125)
        self.assertRaisesRegex(ValueError,
                               "max_features must be in \(0, 1.0\]",
                               model.fit, self.x, self.y)

        model.set_params(max_features="fail_me")
        self.assertRaisesRegex(ValueError,
                               "Invalid value for max_features: 'fail_me'. "
                               "Allowed string values are 'auto', 'sqrt' "
                               "or 'log2'",
                               model.fit, self.x, self.y)

    def test_fit_verbose(self):
        model = GradientBoostingSurvivalAnalysis(n_estimators=10, verbose=1, random_state=0)
        model.fit(self.x, self.y)

    def test_ipcwls_loss(self):
        model = GradientBoostingSurvivalAnalysis(loss="ipcwls", n_estimators=100, max_depth=3, random_state=0)
        model.fit(self.x, self.y)

        time_predicted = model.predict(self.x)
        time_true = self.y["lenfol"]
        event_true = self.y["fstat"]

        rmse_all = numpy.sqrt(mean_squared_error(time_true, time_predicted))
        self.assertAlmostEqual(rmse_all, 590.5441693629117)

        rmse_uncensored = numpy.sqrt(mean_squared_error(time_true[event_true], time_predicted[event_true]))
        self.assertAlmostEqual(rmse_uncensored, 392.97741487479743)

    def test_squared_loss(self):
        model = GradientBoostingSurvivalAnalysis(loss="squared", n_estimators=100, max_depth=3, random_state=0)
        model.fit(self.x, self.y)

        time_predicted = model.predict(self.x)
        time_true = self.y["lenfol"]
        event_true = self.y["fstat"]

        rmse_all = numpy.sqrt(mean_squared_error(time_true, time_predicted))
        self.assertAlmostEqual(rmse_all, 580.23345259002951)

        rmse_uncensored = numpy.sqrt(mean_squared_error(time_true[event_true], time_predicted[event_true]))
        self.assertAlmostEqual(rmse_uncensored, 383.10639243317951)

    def test_ipcw_loss_staged_predict(self):
        # Test whether staged decision function eventually gives
        # the same prediction.
        model = GradientBoostingSurvivalAnalysis(loss="ipcwls", n_estimators=100, max_depth=3, random_state=0)
        model.fit(self.x, self.y)

        y_pred = model.predict(self.x)

        # test if prediction for last stage equals ``predict``
        for y in model.staged_predict(self.x):
            self.assertTupleEqual(y.shape, y_pred.shape)

        assert_array_equal(y_pred, y)

        model.set_params(dropout_rate=0.03)
        model.fit(self.x, self.y)

        y_pred = model.predict(self.x)

        # test if prediction for last stage equals ``predict``
        for y in model.staged_predict(self.x):
            self.assertTupleEqual(y.shape, y_pred.shape)

        assert_array_equal(y_pred, y)

    def test_squared_loss_staged_predict(self):
        # Test whether staged decision function eventually gives
        # the same prediction.
        model = GradientBoostingSurvivalAnalysis(loss="squared", n_estimators=100, max_depth=3, random_state=0)
        model.fit(self.x, self.y)

        y_pred = model.predict(self.x)

        # test if prediction for last stage equals ``predict``
        for y in model.staged_predict(self.x):
            self.assertTupleEqual(y.shape, y_pred.shape)

        assert_array_equal(y_pred, y)

        model.set_params(dropout_rate=0.03)
        model.fit(self.x, self.y)

        y_pred = model.predict(self.x)

        # test if prediction for last stage equals ``predict``
        for y in model.staged_predict(self.x):
            self.assertTupleEqual(y.shape, y_pred.shape)

        assert_array_equal(y_pred, y)

    def test_monitor_early_stopping(self):
        est = GradientBoostingSurvivalAnalysis(loss="ipcwls", n_estimators=50, max_depth=1,
                                               subsample=0.5,
                                               random_state=0)
        est.fit(self.x, self.y, monitor=early_stopping_monitor)

        self.assertEqual(est.n_estimators, 50)  # this is not altered
        self.assertEqual(est.estimators_.shape[0], 10)
        self.assertEqual(est.train_score_.shape[0], 10)
        self.assertEqual(est.oob_improvement_.shape[0], 10)


class TestSparseGradientBoosting(TestCase):

    def setUp(self):
        x, self.y = load_whas500()
        self.x_dense = column.categorical_to_numeric(x.select_dtypes(exclude=[numpy.float_]))

        data = []
        index_i = []
        index_j = []
        for j, (_, col) in enumerate(self.x_dense.iteritems()):
            idx = numpy.flatnonzero(col.values)
            data.extend([1] * len(idx))
            index_i.extend(idx)
            index_j.extend([j] * len(idx))

        self.x_sparse = coo_matrix((data, (index_i, index_j)))
        assert_array_equal(self.x_dense.values, self.x_sparse.toarray())

    def test_fit(self):
        for loss in ('coxph', 'squared', 'ipcwls'):
            model = GradientBoostingSurvivalAnalysis(loss=loss, n_estimators=100, max_depth=1, min_samples_split=10,
                                                     subsample=0.5, random_state=0)
            model.fit(self.x_sparse, self.y)

            self.assertEqual(model.estimators_.shape[0], 100)
            self.assertTupleEqual(model.train_score_.shape, (100,))
            self.assertTupleEqual(model.oob_improvement_.shape, (100,))

            sparse_predict = model.predict(self.x_dense)

            model.fit(self.x_dense, self.y)
            dense_predict = model.predict(self.x_dense)

            assert_array_almost_equal(sparse_predict, dense_predict)

    @attr('slow')
    def test_dropout(self):
        for loss in ('coxph', 'squared', 'ipcwls'):
            model = GradientBoostingSurvivalAnalysis(loss=loss, n_estimators=100, max_depth=1, min_samples_split=10,
                                                     dropout_rate=0.03, random_state=0)
            model.fit(self.x_sparse, self.y)

            self.assertEqual(model.estimators_.shape[0], 100)
            self.assertTupleEqual(model.train_score_.shape, (100,))

            sparse_predict = model.predict(self.x_dense)

            model.fit(self.x_dense, self.y)
            dense_predict = model.predict(self.x_dense)

            assert_array_almost_equal(sparse_predict, dense_predict)


class TestComponentwiseGradientBoosting(TestCase):
    def setUp(self):
        x, self.y = load_whas500()

        x = column.categorical_to_numeric(column.standardize(x, with_std=False))
        self.x = x.values
        self.columns = x.columns.tolist()

    def test_fit(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(n_estimators=100)
        model.fit(self.x, self.y)
        p = model.predict(self.x)

        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        expected_cindex = numpy.array([0.7755659, 58283, 16866, 0, 119])
        assert_array_almost_equal(expected_cindex, numpy.array(result))

        coef_index = ['(Intercept)'] + self.columns

        expected_coef = pandas.Series(numpy.zeros(15, dtype=float), index=coef_index)
        expected_coef['age'] = 0.040919
        expected_coef['hr'] = 0.004977
        expected_coef['diasbp'] = -0.003407
        expected_coef['bmi'] = -0.017938
        expected_coef['sho'] = 0.429904
        expected_coef['chf'] = 0.508211

        assert_array_almost_equal(expected_coef.values, model.coef_)

        self.assertTupleEqual((100,), model.train_score_.shape)

        self.assertRaisesRegex(ValueError, 'Dimensions of X are inconsistent with training data: '
                                           'expected 14 features, but got 2',
                               model.predict, self.x[:, :2])

    def test_fit_subsample(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(n_estimators=100, subsample=0.6, random_state=0)
        model.fit(self.x, self.y)
        p = model.predict(self.x)

        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        expected_cindex = numpy.array([0.7750602, 58245, 16904, 0, 119])
        assert_array_almost_equal(expected_cindex, numpy.array(result))

        coef_index = ['(Intercept)'] + self.columns

        expected_coef = pandas.Series(numpy.zeros(15, dtype=float), index=coef_index)
        expected_coef['age'] = 0.041299
        expected_coef['hr'] = 0.00487
        expected_coef['diasbp'] = -0.003381
        expected_coef['bmi'] = -0.017018
        expected_coef['sho'] = 0.433685
        expected_coef['chf'] = 0.510277

        assert_array_almost_equal(expected_coef.values, model.coef_)

        self.assertTupleEqual((100,), model.train_score_.shape)
        self.assertTupleEqual((100,), model.oob_improvement_.shape)

        self.assertRaisesRegex(ValueError, 'Dimensions of X are inconsistent with training data: '
                                           'expected 14 features, but got 2',
                               model.predict, self.x[:, :2])

    def test_fit_dropout(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(n_estimators=100, learning_rate=1.0,
                                                              dropout_rate=0.03, random_state=0)
        model.fit(self.x, self.y)
        p = model.predict(self.x)

        result = concordance_index_censored(self.y['fstat'], self.y['lenfol'], p)
        expected_cindex = numpy.array([0.7772425, 58409, 16740, 0, 119])
        assert_array_almost_equal(expected_cindex, numpy.array(result))

        coef_index = ['(Intercept)'] + self.columns

        expected_coef = pandas.Series(numpy.zeros(15, dtype=float), index=coef_index)
        expected_coef['age'] = 0.275537
        expected_coef['hr'] = 0.040048
        expected_coef['diasbp'] = -0.029998
        expected_coef['bmi'] = -0.138909
        expected_coef['sho'] = 3.318941
        expected_coef['chf'] = 2.851386
        expected_coef['mitype'] = -0.075817

        assert_array_almost_equal(expected_coef.values, model.coef_)

    def test_feature_importances(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(n_estimators=100, random_state=0)
        model.fit(self.x, self.y)

        self.assertEqual(self.x.shape[1] + 1, len(model.feature_importances_))

    def test_fit_verbose(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(n_estimators=10, verbose=1, random_state=0)
        model.fit(self.x, self.y)

    def test_ipcwls_loss(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(loss="ipcwls", n_estimators=100, random_state=0)
        model.fit(self.x, self.y)

        time_predicted = model.predict(self.x)
        time_true = self.y["lenfol"]
        event_true = self.y["fstat"]

        rmse_all = numpy.sqrt(mean_squared_error(time_true, time_predicted))
        self.assertAlmostEqual(rmse_all, 806.283308322)

        rmse_uncensored = numpy.sqrt(mean_squared_error(time_true[event_true], time_predicted[event_true]))
        self.assertAlmostEqual(rmse_uncensored, 542.884585289)

    def test_squared_loss(self):
        model = ComponentwiseGradientBoostingSurvivalAnalysis(loss="squared", n_estimators=100, random_state=0)
        model.fit(self.x, self.y)

        time_predicted = model.predict(self.x)
        time_true = self.y["lenfol"]
        event_true = self.y["fstat"]

        rmse_all = numpy.sqrt(mean_squared_error(time_true, time_predicted))
        self.assertAlmostEqual(rmse_all, 793.6256945839657)

        rmse_uncensored = numpy.sqrt(mean_squared_error(time_true[event_true], time_predicted[event_true]))
        self.assertAlmostEqual(rmse_uncensored, 542.83358120153525)


class ExceptionCases:
    def test_n_estimators(self):
        model = self.ESTIMATOR(n_estimators=0)

        x = numpy.arange(100).reshape(5, 20)
        y = numpy.empty(dtype=[('event', bool), ('time', float)], shape=5)
        y['time'] = [12, 14, 6, 9, 1]
        y['event'] = [False, False, True, True, False]

        self.assertRaisesRegex(ValueError, "n_estimators must be greater than 0 but was 0",
                               model.fit, x, y)

        model.set_params(n_estimators=-1)
        self.assertRaisesRegex(ValueError, "n_estimators must be greater than 0 but was -1",
                               model.fit, x, y)

    def test_learning_rate(self):
        model = self.ESTIMATOR(learning_rate=0)

        x = numpy.arange(100).reshape(5, 20)
        y = numpy.empty(dtype=[('event', bool), ('time', float)], shape=5)
        y['time'] = [12, 14, 6, 9, 1]
        y['event'] = [False, False, True, True, False]

        self.assertRaisesRegex(ValueError, "learning_rate must be within ]0; 1] but was 0",
                               model.fit, x, y)

        model.set_params(learning_rate=1.2)
        self.assertRaisesRegex(ValueError, "learning_rate must be within ]0; 1] but was 1.2",
                               model.fit, x, y)

    def test_subsample(self):
        model = self.ESTIMATOR(subsample=0)

        x = numpy.arange(100).reshape(5, 20)
        y = numpy.empty(dtype=[('event', bool), ('time', float)], shape=5)
        y['time'] = [12, 14, 6, 9, 1]
        y['event'] = [False, False, True, True, False]

        self.assertRaisesRegex(ValueError, "subsample must be in ]0; 1] but was 0",
                               model.fit, x, y)

        model.set_params(subsample=1.2)
        self.assertRaisesRegex(ValueError, "subsample must be in ]0; 1] but was 1.2",
                               model.fit, x, y)

    def test_dropout_rate(self):
        model = self.ESTIMATOR(dropout_rate=-0.1)

        x = numpy.arange(100).reshape(5, 20)
        y = numpy.empty(dtype=[('event', bool), ('time', float)], shape=5)
        y['time'] = [12, 14, 6, 9, 1]
        y['event'] = [False, False, True, True, False]

        self.assertRaisesRegex(ValueError, "dropout_rate must be within \[0; 1\[, but was -0.1",
                               model.fit, x, y)

        model.set_params(dropout_rate=1.2)
        self.assertRaisesRegex(ValueError, "dropout_rate must be within \[0; 1\[, but was 1.2",
                               model.fit, x, y)

    def test_sample_weight(self):
        model = self.ESTIMATOR()

        x = numpy.arange(100).reshape(5, 20)
        y = numpy.empty(dtype=[('event', bool), ('time', float)], shape=5)
        y['time'] = [12, 14, 6, 9, 1]
        y['event'] = [False, False, True, True, False]

        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: \[5, 3\]",
                               model.fit, x, y, [2, 3, 4])

        model.set_params(dropout_rate=1.2)
        self.assertRaisesRegex(ValueError, "Found input variables with inconsistent numbers of samples: \[5, 8\]",
                               model.fit, x, y, [2, 4, 5, 6, 7, 1, 2, 7])



class TestGradientBoostingSurvivalAnalysisExceptions(TestCase, ExceptionCases):
    ESTIMATOR = GradientBoostingSurvivalAnalysis


class TestComponentwiseGradientBoostingSurvivalAnalysisExceptions(TestCase, ExceptionCases):
    ESTIMATOR = ComponentwiseGradientBoostingSurvivalAnalysis


if __name__ == '__main__':
    run_module_suite()



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import warnings

import numpy
from scipy.linalg import solve
from sklearn.base import BaseEstimator
from sklearn.utils.extmath import squared_norm
from sklearn.utils.validation import check_is_fitted

from ..base import SurvivalAnalysisMixin
from ..functions import StepFunction
from ..nonparametric import _compute_counts
from ..util import check_arrays_survival

__all__ = ['CoxPHSurvivalAnalysis']


class CoxPHOptimizer:
    """Negative partial log-likelihood of Cox proportional hazards model"""

    def __init__(self, X, event, time, alpha):
        # sort descending
        o = numpy.argsort(-time, kind="mergesort")
        self.x = X[o, :]
        self.event = event[o]
        self.time = time[o]
        self.alpha = alpha

    def nlog_likelihood(self, w):
        """Compute negative partial log-likelihood

        Parameters
        ----------
        w : array, shape = [n_features]
            Estimate of coefficients

        Returns
        -------
        loss : float
            Average negative partial log-likelihood
        """
        time = self.time
        n_samples = self.x.shape[0]
        xw = numpy.dot(self.x, w)

        loss = 0
        risk_set = 0
        k = 0
        for i in range(n_samples):
            ti = time[i]
            while k < n_samples and ti == time[k]:
                risk_set += numpy.exp(xw[k])
                k += 1

            if self.event[i]:
                loss -= (xw[i] - numpy.log(risk_set)) / n_samples

        # add regularization term to log-likelihood
        return loss + self.alpha * squared_norm(w) / (2. * n_samples)

    def update(self, w, offset=0):
        """Compute gradient and Hessian matrix with respect to `w`."""
        time = self.time
        x = self.x
        exp_xw = numpy.exp(offset + numpy.dot(x, w))
        n_samples, n_features = x.shape

        gradient = numpy.zeros((1, n_features), dtype=float)
        hessian = numpy.zeros((n_features, n_features), dtype=float)

        inv_n_samples = 1. / n_samples
        risk_set = 0
        risk_set_x = 0
        risk_set_xx = 0
        k = 0
        # iterate time in descending order
        for i in range(n_samples):
            ti = time[i]
            while k < n_samples and ti == time[k]:
                risk_set += exp_xw[k]

                # preserve 2D shape of row vector
                xk = x[k:k + 1]
                risk_set_x += exp_xw[k] * xk

                # outer product
                xx = numpy.dot(xk.T, xk)
                risk_set_xx += exp_xw[k] * xx

                k += 1

            if self.event[i]:
                gradient -= (x[i:i + 1] - risk_set_x / risk_set) * inv_n_samples

                a = risk_set_xx / risk_set
                z = risk_set_x / risk_set
                # outer product
                b = numpy.dot(z.T, z)

                hessian += (a - b) * inv_n_samples

        if self.alpha > 0:
            gradient += self.alpha * inv_n_samples * w

            diag_idx = numpy.diag_indices(n_features)
            hessian[diag_idx] += self.alpha * inv_n_samples

        self.gradient = gradient.ravel()
        self.hessian = hessian


class CoxPHSurvivalAnalysis(BaseEstimator, SurvivalAnalysisMixin):
    """Cox proportional hazards model.

    Uses the Breslow method to handle ties and Newton-Raphson optimization.

    Parameters
    ----------
    alpha : float, optional, default = 0
        Regularization parameter for ridge regression penalty.

    n_iter : int, optional, default = 100
        Maximum number of iterations.

    tol : float, optional, default = 1e-9
        Convergence criteria. Convergence is based on the negative log-likelihood::

        |1 - (new neg. log-likelihood / old neg. log-likelihood) | < tol

    verbose : bool, optional, default = False
        Whether to print statistics on the convergence
        of the optimizer.

    Attributes
    ----------
    `coef_` : ndarray, shape = [n_features]
        Coefficients of the model

    `cum_baseline_hazard_` : :class:`sksurv.functions.StepFunction`
        Estimated baseline cumulative hazard function.

    `baseline_survival_` : :class:`sksurv.functions.StepFunction`
        Estimated baseline survival function.

    References
    ----------
    .. [1] Cox, D. R. Regression models and life tables (with discussion).
           Journal of the Royal Statistical Society. Series B, 34, 187-220, 1972.
    """

    def __init__(self, alpha=0, n_iter=100, tol=1e-9, verbose=False):
        self.alpha = alpha
        self.n_iter = n_iter
        self.tol = tol
        self.verbose = verbose

    def fit(self, X, y):
        """Minimize negative partial log-likelihood for provided data.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix

        y : structured array, shape = [n_samples]
            A structured array containing the binary event indicator
            as first field, and time of event or time of censoring as
            second field.
        """
        X, event, time = check_arrays_survival(X, y)

        if self.alpha < 0:
            raise ValueError("alpha must be positive, but was %r" % self.alpha)

        optimizer = CoxPHOptimizer(X, event, time, self.alpha)

        w = numpy.zeros(X.shape[1])
        w_prev = w
        i = 0
        loss = float('inf')
        while True:
            optimizer.update(w)
            delta = solve(optimizer.hessian, optimizer.gradient,
                          overwrite_a=False, overwrite_b=False, check_finite=False)

            if not numpy.all(numpy.isfinite(delta)):
                raise ValueError("search direction contains NaN or infinite values")

            w_new = w - delta
            loss_new = optimizer.nlog_likelihood(w_new)
            if loss_new > loss:
                # perform step-halving if negative log-likelihood does not decrease
                w = (w_prev + w) / 2
                loss = optimizer.nlog_likelihood(w)
                i += 1
                continue

            w_prev = w
            w = w_new
            i += 1

            res = numpy.abs(1 - (loss_new / loss))
            if res < self.tol:
                break
            elif i >= self.n_iter:
                warnings.warn(('Optimization did not converge: Maximum number of iterations has been exceeded.'),
                              stacklevel=2)
                break

            loss = loss_new

        if self.verbose:
            print("Optimization stopped after %d iterations" % i)

        self.coef_ = w
        self.cum_baseline_hazard_ = self._fit_baseline_hazard_function(X, event, time)
        self.baseline_survival_ = StepFunction(self.cum_baseline_hazard_.x,
                                               numpy.exp(- self.cum_baseline_hazard_.y))

        return self

    def predict(self, X):
        """Predict risk scores.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix.

        Returns
        -------
        risk_score : array, shape = [n_samples]
            Predicted risk scores.
        """
        check_is_fitted(self, "coef_")

        X = numpy.atleast_2d(X)

        return numpy.dot(X, self.coef_)

    def predict_cumulative_hazard_function(self, X):
        """Predict cumulative hazard function.

        The cumulative hazard function for an individual
        with feature vector :math:`x` is defined as

        .. math::

            H(t \\mid x) = \\exp(x^\\top \\beta) H_0(t) ,

        where :math:`H_0(t)` is the baseline hazard function,
        estimated by Breslow's estimator.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix.

        Returns
        -------
        cum_hazard : ndarray, shape = [n_samples]
            Predicted cumulative hazard functions.
        """
        risk_score = numpy.exp(self.predict(X))
        n_samples = risk_score.shape[0]
        funcs = numpy.empty(n_samples, dtype=numpy.object_)
        for i in range(n_samples):
            funcs[i] = StepFunction(x=self.cum_baseline_hazard_.x,
                                    y=self.cum_baseline_hazard_.y,
                                    a=risk_score[i])
        return funcs

    def predict_survival_function(self, X):
        """Predict survival function.

        The survival function for an individual
        with feature vector :math:`x` is defined as

        .. math::

            S(t \\mid x) = S_0(t)^{\\exp(x^\\top \\beta)} ,

        where :math:`S_0(t)` is the baseline survival function,
        estimated by Breslow's estimator.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix.

        Returns
        -------
        survival : ndarray, shape = [n_samples]
            Predicted survival functions.
        """
        risk_score = numpy.exp(self.predict(X))
        n_samples = risk_score.shape[0]
        funcs = numpy.empty(n_samples, dtype=numpy.object_)
        for i in range(n_samples):
            funcs[i] = StepFunction(x=self.baseline_survival_.x,
                                    y=numpy.power(self.baseline_survival_.y, risk_score[i]))
        return funcs

    def _fit_baseline_hazard_function(self, X, event, time):
        """Compute baseline cumulative hazard function.

        Uses Breslow's estimator.

        Parameters
        ----------
        X : array-like, shape = [n_samples, n_features]
            Data matrix.

        event : array-like, shape = [n_samples,]
            Contains binary event indicators.

        time : array-like, shape = [n_samples,]
            Contains event/censoring times.

        Returns
        -------
        cum_baseline_hazard: :class:`sksurv.functions.StepFunction`
            Cumulative baseline hazard function.
        """
        risk_score = numpy.exp(numpy.dot(X, self.coef_))
        order = numpy.argsort(time, kind="mergesort")
        risk_score = risk_score[order]
        uniq_times, n_events, n_at_risk = _compute_counts(event, time, order)

        divisor = numpy.empty(n_at_risk.shape, dtype=numpy.float_)
        value = numpy.sum(risk_score)
        divisor[0] = value
        k = 0
        for i in range(1, len(n_at_risk)):
            d = n_at_risk[i - 1] - n_at_risk[i]
            value -= risk_score[k:(k + d)].sum()
            k += d
            divisor[i] = value

        assert k == n_at_risk[0] - n_at_risk[-1]

        y = numpy.cumsum(n_events / divisor)
        return StepFunction(uniq_times, y)
